@@ -21,6 +21,7 @@
 
 #include <memory>
 #include <iosfwd>
+#include <string>
 
 #include "eckit/distributed/Transport.h"
 
@@ -33,6 +34,7 @@
 #include "fdb5/api/helpers/StatusIterator.h"
 #include "fdb5/api/helpers/WipeIterator.h"
 #include "fdb5/api/helpers/MoveIterator.h"
+#include "fdb5/api/helpers/AxesIterator.h"
 #include "fdb5/config/Config.h"
 #include "fdb5/api/helpers/Callback.h"
 
@@ -94,7 +96,7 @@ public: // methods
 
     ListIterator inspect(const metkit::mars::MarsRequest& request);
 
-    ListIterator list(const FDBToolRequest& request, bool deduplicate=false);
+    ListIterator list(const FDBToolRequest& request, bool deduplicate=false, int level=3);
 
     DumpIterator dump(const FDBToolRequest& request, bool simple=false);
 
@@ -115,6 +117,8 @@ public: // methods
 
     IndexAxis axes(const FDBToolRequest& request, int level=3);
 
+    AxesIterator axesIterator(const FDBToolRequest& request, int level=3);
+
     bool enabled(const ControlIdentifier& controlIdentifier) const;
 
     bool dirty() const;
@@ -155,8 +159,6 @@ private: // members
     bool reportStats_;
 
     FDBStats stats_;
-
-    FlushCallback flushCallback_ = CALLBACK_FLUSH_NOOP;
 };
 
 //----------------------------------------------------------------------------------------------------------------------
