@@ -41,7 +41,7 @@ public: // methods
 
     ListIterator inspect(const metkit::mars::MarsRequest& request) override;
 
-    ListIterator list(const FDBToolRequest& request) override;
+    ListIterator list(const FDBToolRequest& request, int level) override;
 
     DumpIterator dump(const FDBToolRequest& request, bool simple) override;
 
@@ -59,18 +59,20 @@ public: // methods
 
     MoveIterator move(const FDBToolRequest& request, const eckit::URI& dest) override;
 
-    AxesIterator axes(const FDBToolRequest& request, int axes) override;
+    AxesIterator axesIterator(const FDBToolRequest& request, int axes) override;
 
     void flush() override;
 
+protected: // methods
+
+    template <typename VisitorType, typename ... Ts>
+    APIIterator<typename VisitorType::ValueType> queryInternal(const FDBToolRequest& request, Ts ... args);
+
 private: // methods
 
     void print(std::ostream& s) const override;
 
-    template <typename VisitorType, typename ... Ts>
-    APIIterator<typename VisitorType::ValueType> queryInternal(const FDBToolRequest& request, Ts ... args);
-
-private: // members
+protected: // members
 
     std::string home_;
 
