@@ -50,7 +50,7 @@ public: // methods
 
     ListIterator inspect(const metkit::mars::MarsRequest& request) override;
 
-    ListIterator list(const FDBToolRequest& request) override;
+    ListIterator list(const FDBToolRequest& request, int level) override;
 
     DumpIterator dump(const FDBToolRequest& request, bool simple) override;
 
@@ -68,6 +68,8 @@ public: // methods
     
     MoveIterator move(const FDBToolRequest& request, const eckit::URI& dest) override { NOTIMP; }
 
+    AxesIterator axesIterator(const FDBToolRequest& request, int level) override;
+
     void flush() override;
 
 private: // methods
