@@ -31,6 +31,10 @@ void AxesElement::print(std::ostream& out) const {
     out << "Axes(db=" << dbKey_ << ", axes=" << axes_ << ")";
 }
 
+size_t AxesElement::encodeSize() const {
+    return dbKey_.encodeSize() + axes_.encodeSize(IndexAxis::currentVersion());
+}
+
 void AxesElement::encode(eckit::Stream& s) const {
     s << dbKey_;
     axes_.encode(s, IndexAxis::currentVersion());
