@@ -76,7 +76,7 @@ std::vector<FDBToolRequest> FDBToolRequest::requestsFromString(const std::string
                 }
             }*/
             LOG_DEBUG_LIB(LibFdb5) << "Expanded request: " << request << std::endl;
-            requests.emplace_back(FDBToolRequest(request, false, minimumKeys));
+            requests.emplace_back(request, false, minimumKeys);
         }
     }
 
@@ -132,4 +132,3 @@ void FDBToolRequest::checkMinimumKeys(const metkit::mars::MarsRequest& request,
 //----------------------------------------------------------------------------------------------------------------------
 
 } // namespace fdb5
-
