@@ -25,7 +25,6 @@ namespace local {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-// bool StatsVisitor::visitDatabase(const Catalogue& catalogue, const Store& store) {
 bool StatsVisitor::visitDatabase(const Catalogue& catalogue) {
 
     EntryVisitor::visitDatabase(catalogue);
@@ -48,8 +47,6 @@ void StatsVisitor::visitDatum(const Field& field, const std::string& keyFingerpr
     internalVisitor_->visitDatum(field, keyFingerprint);
 }
 
-void StatsVisitor::visitDatum(const Field&, const InspectionKey&) { NOTIMP; }
-
 void StatsVisitor::catalogueComplete(const Catalogue& catalogue) {
     internalVisitor_->catalogueComplete(catalogue);
 
