@@ -37,9 +37,8 @@ public:
     bool visitIndexes() override { return false; }
     bool visitEntries() override { return false; }
     bool visitDatabase(const Catalogue& catalogue) override  { queue_.emplace(catalogue); return true; }
-//    bool visitDatabase(const Catalogue& catalogue, const Store& store) override { queue_.emplace(catalogue); return true; }
     bool visitIndex(const Index&) override { NOTIMP; }
-    void visitDatum(const Field&, const InspectionKey&) override { NOTIMP; }
+    void visitDatum(const Field&, const Key&) override { NOTIMP; }
 
     void visitDatum(const Field& field, const std::string& keyFingerprint) override {
         EntryVisitor::visitDatum(field, keyFingerprint);
