@@ -29,7 +29,7 @@ namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-Config::Config() : schemaPath_("") {
+Config::Config() : schemaPath_(""), schemaPathInitialised_(false) {
     userConfig_ = std::make_shared<eckit::LocalConfiguration>(eckit::LocalConfiguration());
 }
 
@@ -43,8 +43,7 @@ Config Config::make(const eckit::PathName& path, const eckit::Configuration& use
 }
 
 Config::Config(const Configuration& config, const eckit::Configuration& userConfig) :
-    LocalConfiguration(config) {
-    initializeSchemaPath();
+    LocalConfiguration(config), schemaPathInitialised_(false) {
     userConfig_ = std::make_shared<eckit::LocalConfiguration>(userConfig);
 }
 
@@ -142,14 +141,27 @@ PathName Config::expandPath(const std::string& path) const {
 }
 
 const PathName& Config::schemaPath() const {
-    if (schemaPath_.path().empty()) {
+    if (schemaPath_.path().empty() || !schemaPathInitialised_) {
         initializeSchemaPath();
     }
     return schemaPath_;
 }
 
+void Config::overrideSchema(const eckit::PathName& schemaPath, Schema* schema) {
+    ASSERT(schema);
+
+    schema->path_ = schemaPath;
+    SchemaRegistry::instance().add(schemaPath, schema);
+
+    schemaPath_=schemaPath;
+    schemaPathInitialised_ = true;
+}
+
 void Config::initializeSchemaPath() const {
 
+    if (schemaPathInitialised_) {
+        return;
+    }
     // If the user has specified the schema location in the FDB config, use that,
     // otherwise use the library-wide schema path.
 
@@ -165,6 +177,7 @@ void Config::initializeSchemaPath() const {
         schemaPath_ = expandPath(fdbSchemaFile);
     }
 
+    schemaPathInitialised_ = true;
     LOG_DEBUG_LIB(LibFdb5) << "Using FDB schema: " << schemaPath_ << std::endl;
 }
 
@@ -173,6 +186,7 @@ PathName Config::configPath() const {
 }
 
 const Schema& Config::schema() const {
+    initializeSchemaPath();
     return SchemaRegistry::instance().get(schemaPath());
 }
 
