@@ -16,13 +16,13 @@
 #ifndef fdb5_Archiver_H
 #define fdb5_Archiver_H
 
-#include <time.h>
 #include <utility>
 
 #include "eckit/memory/NonCopyable.h"
 
-#include "fdb5/database/DB.h"
+#include "fdb5/api/helpers/Callback.h"
 #include "fdb5/config/Config.h"
+#include "fdb5/database/Catalogue.h"
 
 namespace eckit   {
 class DataHandle;
@@ -38,46 +38,50 @@ class Schema;
 
 //----------------------------------------------------------------------------------------------------------------------
 
+struct Database {
+    time_t                              time_;
+    std::unique_ptr<CatalogueWriter>    catalogue_;
+    std::unique_ptr<Store>              store_;
+};
 class Archiver : public eckit::NonCopyable {
 
 public: // methods
 
-    Archiver(const Config& dbConfig = Config().expandConfig(), const ArchiveCallback& callback = CALLBACK_NOOP);
+    Archiver(const Config& dbConfig = Config().expandConfig(), const ArchiveCallback& callback = CALLBACK_ARCHIVE_NOOP);
 
     virtual ~Archiver();
 
-    void archive(const Key &key, BaseArchiveVisitor& visitor);
-    void archive(const Key &key, const void* data, size_t len);
+    void archive(const Key& key, BaseArchiveVisitor& visitor);
+    void archive(const Key& key, const void* data, size_t len);
 
     /// Flushes all buffers and closes all data handles into a consistent DB state
     /// @note always safe to call
     void flush();
 
-    friend std::ostream &operator<<(std::ostream &s, const Archiver &x) {
+    friend std::ostream& operator<<(std::ostream& s, const Archiver& x) {
         x.print(s);
         return s;
     }
 
 private: // methods
 
-    void print(std::ostream &out) const;
+    void print(std::ostream& out) const;
 
-    DB& database(const Key &key);
+    void selectDatabase(const Key& key);
 
 private: // members
-
     friend class BaseArchiveVisitor;
 
-    typedef std::map< Key, std::pair<time_t, std::unique_ptr<DB> > > store_t;
-
     Config dbConfig_;
 
-    store_t databases_;
+    std::map<Key, Database> databases_;
 
     std::vector<Key> prev_;
 
-    DB* current_;
+    Database* db_;
 
+    std::recursive_mutex flushMutex_;
+    std::mutex cacheMutex_;
     const ArchiveCallback& callback_;
 };
 
