@@ -14,43 +14,47 @@
 #include "fdb5/database/Archiver.h"
 #include "fdb5/database/BaseArchiveVisitor.h"
 #include "fdb5/rules/Rule.h"
+#include "fdb5/database/Store.h"
 
 namespace fdb5 {
 
-BaseArchiveVisitor::BaseArchiveVisitor(Archiver &owner, const Key &field) :
+BaseArchiveVisitor::BaseArchiveVisitor(Archiver &owner, const Key& initialFieldKey) :
     WriteVisitor(owner.prev_),
     owner_(owner),
-    field_(field) {
+    initialFieldKey_(initialFieldKey) {
     checkMissingKeysOnWrite_ = eckit::Resource<bool>("checkMissingKeysOnWrite", true);
 }
 
-bool BaseArchiveVisitor::selectDatabase(const Key &key, const Key&) {
-    LOG_DEBUG_LIB(LibFdb5) << "selectDatabase " << key << std::endl;
-    owner_.current_ = &owner_.database(key);
-    owner_.current_->deselectIndex();
+bool BaseArchiveVisitor::selectDatabase(const Key& dbKey, const Key&) {
+    LOG_DEBUG_LIB(LibFdb5) << "BaseArchiveVisitor::selectDatabase " << dbKey << std::endl;
+    owner_.selectDatabase(dbKey);
+    catalogue()->deselectIndex();
 
     return true;
 }
 
-bool BaseArchiveVisitor::selectIndex(const Key &key, const Key&) {
-    // eckit::Log::info() << "selectIndex " << key << std::endl;
-    ASSERT(owner_.current_);
-    return owner_.current_->selectIndex(key);
+bool BaseArchiveVisitor::selectIndex(const Key& idxKey, const Key&) {
+    return catalogue()->selectIndex(idxKey);
 }
 
-void BaseArchiveVisitor::checkMissingKeys(const Key &full) {
-    if (checkMissingKeysOnWrite_) {
-        field_.validateKeysOf(full);
-    }
+void BaseArchiveVisitor::checkMissingKeys(const Key& fullKey) const {
+    if (checkMissingKeysOnWrite_) { fullKey.validateKeys(initialFieldKey_); }
 }
 
 const Schema& BaseArchiveVisitor::databaseSchema() const {
-    ASSERT(current());
-    return current()->schema();
+    return catalogue()->schema();
 }
 
-DB* BaseArchiveVisitor::current() const {
-    return owner_.current_;
+CatalogueWriter* BaseArchiveVisitor::catalogue() const {
+    ASSERT(owner_.db_);
+    ASSERT(owner_.db_->catalogue_);
+    return owner_.db_->catalogue_.get();
+}
+
+Store* BaseArchiveVisitor::store() const {
+    ASSERT(owner_.db_);
+    ASSERT(owner_.db_->store_);
+    return owner_.db_->store_.get();
 }
 
 //----------------------------------------------------------------------------------------------------------------------
