@@ -25,8 +25,6 @@
 #include "eckit/memory/NonCopyable.h"
 #include "eckit/filesystem/URI.h"
 
-#include "fdb5/database/DB.h"
-
 
 namespace fdb5 {
 
@@ -53,18 +51,12 @@ public: // methods
     virtual bool canHandle(const eckit::URI& uri, const Config&) const = 0;
 
     /// Uniquely selects a location where the Key will be put or already exists
-    virtual eckit::URI location(const Key &key, const Config& config) const = 0;
-
-    /// Lists the roots that can be visited given a DB key
-    virtual std::vector<eckit::URI> allLocations(const Key& key, const Config& config) const = 0;
+    virtual eckit::URI location(const Key& key, const Config& config) const = 0;
 
     /// Lists the roots that can be visited given a DB key
     virtual std::vector<eckit::URI> visitableLocations(const Key& key, const Config& config) const = 0;
     virtual std::vector<eckit::URI> visitableLocations(const metkit::mars::MarsRequest& rq, const Config& config) const = 0;
 
-    /// Lists the roots where a DB key would be able to be written
-    virtual std::vector<eckit::URI> writableLocations(const Key& key, const Config& config) const = 0;
-
     friend std::ostream &operator<<(std::ostream &s, const Engine& x);
 
 protected: // methods
