@@ -8,15 +8,20 @@
  * does it submit to any jurisdiction.
  */
 
-#include "fdb5/database/EntryVisitMechanism.h"
-
+#include "eckit/exception/Exceptions.h"
 #include "eckit/io/AutoCloser.h"
+#include "eckit/log/Log.h"
 
-#include "fdb5/api/helpers/FDBToolRequest.h"
-#include "fdb5/database/Manager.h"
-#include "fdb5/database/Engine.h"
 #include "fdb5/LibFdb5.h"
+#include "fdb5/api/helpers/FDBToolRequest.h"
+#include "fdb5/database/Engine.h"
+#include "fdb5/database/EntryVisitMechanism.h"
+#include "fdb5/database/Manager.h"
 #include "fdb5/rules/Schema.h"
+#include "fdb5/database/Store.h"
+
+#include <memory>
+#include <vector>
 
 using namespace eckit;
 
@@ -32,13 +37,32 @@ public:
 
 //----------------------------------------------------------------------------------------------------------------------
 
-EntryVisitor::EntryVisitor() : currentCatalogue_(nullptr), currentIndex_(nullptr) {}
+bool EntryVisitor::preVisitDatabase(const eckit::URI& /*uri*/, const Schema& /*schema*/) {
+    return true;
+}
 
-EntryVisitor::~EntryVisitor() {}
+EntryVisitor::EntryVisitor() : currentCatalogue_(nullptr), currentStore_(nullptr), currentIndex_(nullptr) {}
 
-bool EntryVisitor::visitDatabase(const Catalogue& catalogue, const Store& store) {
+EntryVisitor::~EntryVisitor() {
+    if (currentStore_) {
+        delete currentStore_;
+    }
+}
+
+Store& EntryVisitor::store() const {
+    if (!currentStore_) {
+        ASSERT(currentCatalogue_);
+        currentStore_ = currentCatalogue_->buildStore().release();
+        ASSERT(currentStore_);
+    }
+    return *currentStore_;
+}
+
+bool EntryVisitor::visitDatabase(const Catalogue& catalogue) {
     currentCatalogue_ = &catalogue;
-    currentStore_ = &store;
+    currentStore_ = nullptr;
+    currentIndex_ = nullptr;
+    rule_ = &currentCatalogue_->rule();
     return true;
 }
 
@@ -47,12 +71,16 @@ void EntryVisitor::catalogueComplete(const Catalogue& catalogue) {
         ASSERT(currentCatalogue_ == &catalogue);
     }
     currentCatalogue_ = nullptr;
+    delete currentStore_;
     currentStore_ = nullptr;
     currentIndex_ = nullptr;
+    rule_ = nullptr;
 }
 
 bool EntryVisitor::visitIndex(const Index& index) {
+    ASSERT(currentCatalogue_);
     currentIndex_ = &index;
+    rule_         = &currentCatalogue_->schema().matchingRule(currentCatalogue_->key(), currentIndex_->key());
     return true;
 }
 
@@ -60,10 +88,10 @@ void EntryVisitor::visitDatum(const Field& field, const std::string& keyFingerpr
     ASSERT(currentCatalogue_);
     ASSERT(currentIndex_);
 
-    Key key(keyFingerprint, currentCatalogue_->schema().ruleFor(currentCatalogue_->key(), currentIndex_->key()));
-    visitDatum(field, key);
-}
+    const auto datumKey = rule_->makeKey(keyFingerprint);
 
+    visitDatum(field, datumKey);
+}
 
 time_t EntryVisitor::indexTimestamp() const {
     return currentIndex_ == nullptr ? 0 : currentIndex_->timestamp();
@@ -85,41 +113,37 @@ void EntryVisitMechanism::visit(const FDBToolRequest& request, EntryVisitor& vis
 
     ASSERT(request.all() == request.request().empty());
 
-    // TODO: Put minimim keys check into FDBToolRequest.
+    /// @todo Put minimim keys check into FDBToolRequest.
 
     LOG_DEBUG_LIB(LibFdb5) << "REQUEST ====> " << request.request() << std::endl;
 
     try {
-
-        fdb5::Manager mg{dbConfig_};
+        fdb5::Manager    mg {dbConfig_};
         std::vector<URI> uris(mg.visitableLocations(request.request(), request.all()));
 
         // n.b. it is not an error if nothing is found (especially in a sub-fdb).
 
         // And do the visitation
+        for (const URI& uri : uris) {
+            if (!visitor.preVisitDatabase(uri, dbConfig_.schema())) { continue; }
 
-        for (URI uri : uris) {
-            /// @note: the schema of a URI returned by visitableLocations 
+            /// @note: the schema of a URI returned by visitableLocations
             ///   matches the corresponding Engine type name
             // fdb5::Engine& ng = fdb5::Engine::backend(uri.scheme());
+            LOG_DEBUG_LIB(LibFdb5) << "FDB processing URI " << uri << std::endl;
 
-            std::unique_ptr<DB> db;
-
+            std::unique_ptr<CatalogueReader> catalogue;
             try {
-                
-                db = DB::buildReader(uri, dbConfig_);
 
-            } catch (fdb5::DatabaseNotFoundException& e) {
+                catalogue = CatalogueReaderFactory::instance().build(uri, dbConfig_);
 
-                visitor.onDatabaseNotFound(e);
+            } catch (fdb5::DatabaseNotFoundException& e) { visitor.onDatabaseNotFound(e); }
 
-            }
+            ASSERT(catalogue->open());
 
-            ASSERT(db->open());
-            eckit::AutoCloser<DB> closer(*db);
-
-            db->visitEntries(visitor, false);
+            eckit::AutoCloser<Catalogue> closer(*catalogue);
 
+            catalogue->visitEntries(visitor, /* *store, */ false);
         }
 
     } catch (eckit::UserError&) {
