@@ -43,7 +43,7 @@ public: // methods
 
     Field();
 
-    Field(std::shared_ptr<FieldLocation> location, time_t timestamp, const FieldDetails& details = FieldDetails());
+    Field(std::shared_ptr<const FieldLocation> location, time_t timestamp, const FieldDetails& details = FieldDetails());
     Field(const FieldLocation&& location, time_t timestamp, const FieldDetails& details = FieldDetails());
 
     eckit::DataHandle* dataHandle() const { return location_->dataHandle(); }
@@ -54,14 +54,14 @@ public: // methods
 
     /// stableLocation is an object with validity that extends longer than that of the
     /// owning DB. May need converting to a more static form --- or not.
-    std::shared_ptr<FieldLocation> stableLocation() const {
+    std::shared_ptr<const FieldLocation> stableLocation() const {
         return location_->stableLocation();
     }
     const FieldDetails& details() const { return details_; }
 
 private: // members
 
-   std::shared_ptr<FieldLocation> location_;
+   std::shared_ptr<const FieldLocation> location_;
 
     time_t timestamp_;
 
