@@ -22,7 +22,7 @@
 
 #include "eckit/container/DenseSet.h"
 #include "eckit/memory/NonCopyable.h"
-#include "eckit/filesystem/PathName.h"
+// #include "eckit/filesystem/PathName.h"
 #include "eckit/types/Types.h"
 
 namespace eckit {
@@ -37,6 +37,8 @@ namespace fdb5 {
 
 class Key;
 
+// class TypesRegistry;
+
 //----------------------------------------------------------------------------------------------------------------------
 
 class IndexAxis : private eckit::NonCopyable {
@@ -54,9 +56,10 @@ public: // methods
     bool operator==(const IndexAxis& rhs) const;
     bool operator!=(const IndexAxis& rhs) const;
 
-    void insert(const Key &key);
+    void insert(const Key& key);
     /// @note: the values are required to be cannonicalised
     void insert(const std::string& axis, const std::vector<std::string>& values);
+    size_t encodeSize(const int version) const;
     void encode(eckit::Stream &s, const int version) const;
     static int currentVersion() { return 3; }
 
@@ -74,6 +77,7 @@ public: // methods
 
     bool partialMatch(const metkit::mars::MarsRequest& request) const;
     bool contains(const Key& key) const;
+    bool containsPartial(const Key& key) const;
 
     /// Provide a means to test if the index has changed since it was last written out, and to
     /// mark that it has been written out.
@@ -109,7 +113,7 @@ private: // methods
 
 private: // members
 
-    typedef std::map<std::string, std::shared_ptr<eckit::DenseSet<std::string> > > AxisMap;
+    typedef std::map<std::string, std::shared_ptr<eckit::DenseSet<std::string>>> AxisMap;
     AxisMap axis_;
 
     bool readOnly_;
