@@ -33,28 +33,24 @@ public: // methods
 
     RetrieveVisitor(const Notifier &wind, HandleGatherer &gatherer);
 
-    ~RetrieveVisitor();
-
-
-private:  // methods
-
+protected:  // methods
     // From Visitor
 
-    virtual bool selectDatabase(const Key &key, const Key &full) override;
+    bool selectDatabase(const Key& dbKey, const Key& fullKey) override;
 
-    virtual bool selectIndex(const Key &key, const Key &full) override;
+    bool selectIndex(const Key& idxKey, const Key& fullKey) override;
 
-    virtual bool selectDatum(const InspectionKey &key, const Key &full) override;
+    bool selectDatum(const Key& datumKey, const Key& fullKey) override;
 
-    virtual void values(const metkit::mars::MarsRequest& request,
+    void values(const metkit::mars::MarsRequest& request,
                         const std::string& keyword,
                         const TypesRegistry& registry,
                         eckit::StringList& values) override;
 
-    virtual void print( std::ostream &out ) const override;
+    void print( std::ostream &out ) const override;
 
     Store& store();
-    virtual const Schema& databaseSchema() const override;
+    const Schema& databaseSchema() const override;
 
 private:
 
