@@ -37,7 +37,9 @@ public: // methods
     ~WipeVisitor() override;
 
     bool visitEntries() override { return false; }
-    void visitDatum(const Field&, const InspectionKey&) override { NOTIMP; }
+
+    void visitDatum(const Field& /*field*/, const Key& /*datumKey*/) override { NOTIMP; }
+
     void visitDatum(const Field& /*field*/, const std::string& /*keyFingerprint*/) override { NOTIMP; }
 
 protected: // members
