@@ -14,15 +14,10 @@ namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-WriteVisitor::WriteVisitor(std::vector<Key> &prev) :
-    prev_(prev),
-    rule_(0) {
+WriteVisitor::WriteVisitor(std::vector<Key>& prev) : prev_(prev) {
     prev.resize(3);
 }
 
-WriteVisitor::~WriteVisitor() {
-}
-
 //----------------------------------------------------------------------------------------------------------------------
 
 } // namespace fdb5
