@@ -15,7 +15,6 @@
 #pragma once
 
 #include <unistd.h>
-#include <stdlib.h>
 #include <thread>
 
 #include "eckit/net/Port.h"
@@ -29,8 +28,6 @@
 #include "fdb5/config/Config.h"
 #include "fdb5/LibFdb5.h"
 
-#include "fdb5/remote/AvailablePortList.h"
-
 namespace fdb5::remote {
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -41,11 +38,11 @@ class FDBForker : public eckit::ProcessControler {
 public: // methods
 
     FDBForker(eckit::net::TCPSocket& socket, const Config& config);
-    virtual ~FDBForker() override;
+    ~FDBForker() override;
 
 private: // methods
 
-    virtual void run() override;
+    void run() override;
 
     eckit::net::TCPSocket socket_;
     eckit::LocalConfiguration config_;
@@ -81,11 +78,11 @@ class FdbServer : public eckit::Application, public FdbServerBase {
 public:
     FdbServer(int argc, char** argv, const char* home);
 
-    virtual ~FdbServer() override;
+    ~FdbServer() override;
 
-    virtual void run() override;
+    void run() override;
 
-    void virtual hookUnique() override; // non-unique
+    void hookUnique() override; // non-unique
 
 };
 
