@@ -37,10 +37,10 @@ public:
     RemoteFieldLocation(eckit::Stream&);
     RemoteFieldLocation(const RemoteFieldLocation&);
 
-    virtual eckit::DataHandle *dataHandle() const override;
+    eckit::DataHandle* dataHandle() const override;
 
-    virtual std::shared_ptr<FieldLocation> make_shared() const override;
-    virtual void visit(FieldLocationVisitor& visitor) const override;
+    std::shared_ptr<const FieldLocation> make_shared() const override;
+    void visit(FieldLocationVisitor& visitor) const override;
 
 public: // For Streamable
 
@@ -48,15 +48,15 @@ public: // For Streamable
 
 protected: // For Streamable
 
-    virtual void encode(eckit::Stream&) const override;
-    virtual const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
+    void encode(eckit::Stream&) const override;
+    const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
 
     static eckit::ClassSpec                       classSpec_;
     static eckit::Reanimator<RemoteFieldLocation> reanimator_;
 
 private: // methods
 
-    virtual void print(std::ostream &out) const override;
+    void print(std::ostream& out) const override;
 
 private: // members
 
