@@ -10,16 +10,17 @@
 
 #pragma once
 
-#include <future>
-
-#include "eckit/config/Configuration.h"
 #include "eckit/memory/NonCopyable.h"
 #include "eckit/net/Endpoint.h"
 
-#include "fdb5/database/Key.h"
+#include "fdb5/remote/Connection.h"
 #include "fdb5/remote/Messages.h"
 #include "fdb5/remote/client/ClientConnection.h"
 
+#include <mutex>
+#include <utility>  // std::pair
+#include <vector>
+
 namespace fdb5::remote {
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -33,35 +34,50 @@ public:
 //----------------------------------------------------------------------------------------------------------------------
 
 class Client : eckit::NonCopyable {
-public:
+public:  // types
+    using PayloadList  = Connection::PayloadList;
+    using EndpointList = std::vector<std::pair<eckit::net::Endpoint, std::string>>;
+
+    static constexpr size_t defaultBufferSizeArchive = 8192;
+    static constexpr size_t defaultBufferSizeFlush   = 1024;
+    static constexpr size_t defaultBufferSizeKey     = 4096;
+
+public:  // methods
     Client(const eckit::net::Endpoint& endpoint, const std::string& defaultEndpoint);
-    Client(const std::vector<std::pair<eckit::net::Endpoint, std::string>>& endpoints);
-    ~Client();
+
+    Client(const EndpointList& endpoints);
+
+    virtual ~Client();
 
     uint32_t clientId() const { return id_; }
+
     uint32_t id() const { return id_; }
+
     const eckit::net::Endpoint& controlEndpoint() const { return connection_.controlEndpoint(); }
+
     const std::string& defaultEndpoint() const { return connection_.defaultEndpoint(); }
 
-    uint32_t generateRequestID() { return connection_.generateRequestID(); }
+    uint32_t generateRequestID() const { return connection_.generateRequestID(); }
 
     // blocking requests
-    void controlWriteCheckResponse(Message msg, uint32_t requestID, bool dataListener, const void* payload=nullptr, uint32_t payloadLength=0);
-    eckit::Buffer controlWriteReadResponse (Message msg, uint32_t requestID, const void* payload=nullptr, uint32_t payloadLength=0);
+    void controlWriteCheckResponse(Message     msg,
+                                   uint32_t    requestID,
+                                   bool        dataListener,
+                                   const void* payload       = nullptr,
+                                   uint32_t    payloadLength = 0) const;
+
+    eckit::Buffer controlWriteReadResponse(Message     msg,
+                                           uint32_t    requestID,
+                                           const void* payload       = nullptr,
+                                           uint32_t    payloadLength = 0) const;
+
+    void dataWrite(Message msg, uint32_t requestID, PayloadList payloads = {});
 
-    void dataWrite(remote::Message msg, uint32_t requestID, std::vector<std::pair<const void*, uint32_t>> data={});
-    
     // handlers for incoming messages - to be defined in the client class
-    virtual bool handle(Message message, bool control, uint32_t requestID) = 0;
-    virtual bool handle(Message message, bool control, uint32_t requestID, eckit::Buffer&& payload) = 0;
-    virtual void handleException(std::exception_ptr e) = 0;
-
-    bool response(uint32_t requestID);
-    bool response(uint32_t requestID, eckit::Buffer&& payload);
-    uint32_t blockingRequestId() { return blockingRequestId_; }
+    virtual bool handle(Message message, uint32_t requestID)                          = 0;
+    virtual bool handle(Message message, uint32_t requestID, eckit::Buffer&& payload) = 0;
 
 protected:
-    
     ClientConnection& connection_;
 
 private:
@@ -70,10 +86,7 @@ private:
 private:
     uint32_t id_;
 
-    std::mutex blockingRequestMutex_;
-    uint32_t blockingRequestId_;
-    std::promise<bool> promise_;
-    std::promise<eckit::Buffer> payloadPromise_;
+    mutable std::mutex blockingRequestMutex_;
 };
 
-}
\ No newline at end of file
+}  // namespace fdb5::remote
