@@ -8,16 +8,24 @@
  * does it submit to any jurisdiction.
  */
 
-#include "eckit/config/Resource.h"
-#include "eckit/net/NetMask.h"
-#include "eckit/serialisation/MemoryStream.h"
-
+#include "fdb5/remote/server/CatalogueHandler.h"
 #include "fdb5/LibFdb5.h"
 #include "fdb5/api/helpers/FDBToolRequest.h"
-#include "fdb5/remote/server/CatalogueHandler.h"
+#include "fdb5/database/Catalogue.h"
+#include "fdb5/database/Key.h"
+#include "fdb5/remote/Connection.h"
+#include "fdb5/remote/Messages.h"
+#include "fdb5/remote/server/ServerConnection.h"
+
+#include "eckit/net/NetMask.h"
+#include "eckit/net/TCPSocket.h"
+#include "eckit/serialisation/MemoryStream.h"
+
+#include <future>
+#include <mutex>
+#include <utility>
 
 using namespace eckit;
-using metkit::mars::MarsRequest;
 
 namespace fdb5::remote {
 
@@ -30,7 +38,7 @@ namespace fdb5::remote {
 // ***************************************************************************************
 
 CatalogueHandler::CatalogueHandler(eckit::net::TCPSocket& socket, const Config& config):
-    ServerConnection(socket, config), fdbId_(0), fdbControlConnection_(false), fdbDataConnection_(false) {}
+    ServerConnection(socket, config), fdbControlConnection_(false), fdbDataConnection_(false) {}
 
 CatalogueHandler::~CatalogueHandler() {}
 
@@ -41,8 +49,9 @@ Handled CatalogueHandler::handleControl(Message message, uint32_t clientID, uint
             case Message::Schema: // request top-level schema
                 {
                     std::lock_guard<std::mutex> lock(handlerMutex_);
-                    if (fdbId_ == 0) {
-                        fdbId_ = clientID;
+                    auto it = fdbs_.find(clientID);
+                    if (it == fdbs_.end()) {
+                        fdbs_[clientID];
                         fdbControlConnection_ = true;
                         fdbDataConnection_ = !single_;
                         numControlConnection_++;
@@ -62,10 +71,6 @@ Handled CatalogueHandler::handleControl(Message message, uint32_t clientID, uint
                 archiver();
                 return Handled::YesAddArchiveListener;
 
-            case Message::Flush: // notification that the client has sent all data locations for archival
-                flush(clientID, requestID, eckit::Buffer{0});
-                return Handled::Yes;
-
             default: {
                 std::stringstream ss;
                 ss << "ERROR: Unexpected message recieved (" << message << "). ABORTING";
@@ -98,14 +103,26 @@ Handled CatalogueHandler::handleControl(Message message, uint32_t clientID, uint
                 list(clientID, requestID, std::move(payload));
                 return Handled::Yes;
 
+            case Message::Axes: // list request. Location are sent aynchronously over the data connection
+                axes(clientID, requestID, std::move(payload));
+                return Handled::Yes;
+
             case Message::Inspect: // inspect request. Location are sent aynchronously over the data connection
                 inspect(clientID, requestID, std::move(payload));
                 return Handled::Yes;
 
+            case Message::Stats: // inspect request. Location are sent aynchronously over the data connection
+                stats(clientID, requestID, std::move(payload));
+                return Handled::Yes;
+
             case Message::Flush: // flush catalogue
                 flush(clientID, requestID, std::move(payload));
                 return Handled::Yes;
 
+            case Message::Exists:  // check if catalogue exists
+                exists(clientID, requestID, std::move(payload));
+                return Handled::Replied;
+
             default: {
                 std::stringstream ss;
                 ss << "ERROR: Unexpected message recieved (" << message << "). ABORTING";
@@ -125,57 +142,6 @@ Handled CatalogueHandler::handleControl(Message message, uint32_t clientID, uint
     return Handled::No;
 }
 
-// Handled CatalogueHandler::handleData(Message message, uint32_t clientID, uint32_t requestID) {
-//     try {
-//         switch (message) {
-//             case Message::Flush: // notification that the client has sent all data locations for archival
-//                 return Handled::YesRemoveArchiveListener; // ????
-//
-//             default: {
-//                 std::stringstream ss;
-//                 ss << "ERROR: Unexpected message recieved (" << message << "). ABORTING";
-//                 Log::status() << ss.str() << std::endl;
-//                 Log::error() << ss.str() << std::endl;
-//                 throw SeriousBug(ss.str(), Here());
-//             }
-//         }
-//     }
-//     catch (std::exception& e) {
-//         // n.b. more general than eckit::Exception
-//         error(e.what(), clientID, requestID);
-//     }
-//     catch (...) {
-//         error("Caught unexpected and unknown error", clientID, requestID);
-//     }
-//     return Handled::No;
-// }
-
-// Handled CatalogueHandler::handleData(Message message, uint32_t clientID, uint32_t requestID, /*eckit::net::Endpoint endpoint,*/ eckit::Buffer&& payload) {
-//     try {
-//         switch (message) {
-//             case Message::Blob:
-//             case Message::MultiBlob:
-//                 queue(message, clientID, requestID, std::move(payload));
-//                 return Handled::Yes;
-//
-//             default: {
-//                 std::stringstream ss;
-//                 ss << "ERROR: Unexpected message recieved (" << message << "). ABORTING";
-//                 Log::status() << ss.str() << std::endl;
-//                 Log::error() << ss.str() << std::endl;
-//                 throw SeriousBug(ss.str(), Here());
-//             }
-//         }
-//     }
-//     catch (std::exception& e) {
-//         // n.b. more general than eckit::Exception
-//         error(e.what(), clientID, requestID);
-//     }
-//     catch (...) {
-//         error("Caught unexpected and unknown error", clientID, requestID);
-//     }
-//     return Handled::No;
-// }
 
 // API forwarding logic, adapted from original remoteHandler
 // Used for Inspect and List
@@ -207,16 +173,36 @@ struct BaseHelper {
 
 struct ListHelper : public BaseHelper<ListElement> {
     ListIterator apiCall(FDB& fdb, const FDBToolRequest& request) const {
+        /// @todo remember to add level_ to this helper
         return fdb.list(request);
     }
 };
 
+struct AxesHelper : public BaseHelper<AxesElement> {
+    virtual size_t encodeBufferSize(const AxesElement& el) const { return el.encodeSize(); }
+
+    void extraDecode(eckit::Stream& s) {
+        s >> level_;
+    }
+    AxesIterator apiCall(FDB& fdb, const FDBToolRequest& request) const {
+        return fdb.axesIterator(request, level_);
+    }
+private:
+    int level_;
+};
+
 struct InspectHelper : public BaseHelper<ListElement> {
     ListIterator apiCall(FDB& fdb, const FDBToolRequest& request) const {
         return fdb.inspect(request.request());
     }
 };
 
+struct StatsHelper : public BaseHelper<StatsElement> {
+    StatsIterator apiCall(FDB& fdb, const FDBToolRequest& request) const {
+        return fdb.stats(request);
+    }
+};
+
 template <typename HelperClass>
 void CatalogueHandler::forwardApiCall(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) {
     HelperClass helper;
@@ -227,18 +213,33 @@ void CatalogueHandler::forwardApiCall(uint32_t clientID, uint32_t requestID, eck
     helper.extraDecode(s);
 
     // Construct worker thread to feed responses back to client
-
     ASSERT(workerThreads_.find(requestID) == workerThreads_.end());
 
+    {
+        std::lock_guard<std::mutex> lock(fdbMutex_);
+        auto it = fdbs_.find(clientID);
+        if (it == fdbs_.end()) {
+            fdbs_[clientID];
+        }
+    }
+
     workerThreads_.emplace(
         requestID, std::async(std::launch::async, [request, clientID, requestID, helper, this]() {
 
             try {
-                auto iterator = helper.apiCall(fdb_, request);
+                FDB* fdb = nullptr;
+                {
+                    std::lock_guard<std::mutex> lock(handlerMutex_);
+                    auto it = fdbs_.find(clientID);
+                    ASSERT(it != fdbs_.end());
+                    fdb = &it->second;
+                    ASSERT(fdb);
+                }
+                auto iterator = helper.apiCall(*fdb, request);
                 typename decltype(iterator)::value_type elem;
                 while (iterator.next(elem)) {
                     auto encoded(helper.encode(elem, *this));
-                    write(Message::Blob, false, clientID, requestID, std::vector<std::pair<const void*, uint32_t>>{{encoded.buf, encoded.position}});
+                    write(Message::Blob, false, clientID, requestID, encoded.buf, encoded.position);
                 }
                 write(Message::Complete, false, clientID, requestID);
             }
@@ -257,10 +258,18 @@ void CatalogueHandler::list(uint32_t clientID, uint32_t requestID, eckit::Buffer
     forwardApiCall<ListHelper>(clientID, requestID, std::move(payload));
 }
 
+void CatalogueHandler::axes(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) {
+    forwardApiCall<AxesHelper>(clientID, requestID, std::move(payload));
+}
+
 void CatalogueHandler::inspect(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) {
     forwardApiCall<InspectHelper>(clientID, requestID, std::move(payload));
 }
 
+void CatalogueHandler::stats(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) {
+    forwardApiCall<StatsHelper>(clientID, requestID, std::move(payload));
+}
+
 void CatalogueHandler::schema(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) {
 
     eckit::Buffer schemaBuffer(256*1024);
@@ -272,14 +281,14 @@ void CatalogueHandler::schema(uint32_t clientID, uint32_t requestID, eckit::Buff
         // 1. Read dbkey to select catalogue
         MemoryStream s(payload);
         Key dbKey(s);
-    
+
         // 2. Get catalogue
         Catalogue& cat = catalogue(clientID, dbKey);
         const Schema& schema = cat.schema();
         stream << schema;
     }
 
-    write(Message::Received, false, clientID, requestID, schemaBuffer.data(), stream.position());
+    write(Message::Received, true, clientID, requestID, schemaBuffer.data(), stream.position());
 }
 
 void CatalogueHandler::stores(uint32_t clientID, uint32_t requestID) {
@@ -299,11 +308,10 @@ void CatalogueHandler::stores(uint32_t clientID, uint32_t requestID) {
         }
     }
 
-    Log::debug<LibFdb5>() << "Client " << clientIPaddress << " from network '" << clientNetwork << "'" << std::endl;
+    LOG_DEBUG_LIB(LibFdb5) << "Client " << clientIPaddress << " from network '" << clientNetwork << "'" << std::endl;
 
     ASSERT(config_.has("stores"));
     std::map<std::string, std::vector<eckit::net::Endpoint>> stores;
-    // std::vector<std::pair<std::string, eckit::net::Endpoint>> stores;
     for (const auto& configStore: config_.getSubConfigurations("stores")) {
         ASSERT(configStore.has("default"));
         eckit::net::Endpoint fieldLocationEndpoint{configStore.getString("default")};
@@ -342,34 +350,62 @@ void CatalogueHandler::stores(uint32_t clientID, uint32_t requestID) {
                 s << ee;
             }
         }
-        write(Message::Received, false, clientID, requestID, startupBuffer.data(), s.position());
+        write(Message::Received, true, clientID, requestID, startupBuffer.data(), s.position());
     }
 }
 
+void CatalogueHandler::exists(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) const {
 
+    ASSERT(payload.size() > 0);
 
+    bool exists = false;
 
+    {
+        eckit::MemoryStream stream(payload);
+        const Key dbKey(stream);
+        exists = CatalogueReaderFactory::instance().build(dbKey, config_)->exists();
+    }
 
+    eckit::Buffer       existBuf(5);
+    eckit::MemoryStream stream(existBuf);
+    stream << exists;
 
+    write(Message::Received, true, clientID, requestID, existBuf.data(), stream.position());
+}
 
 void CatalogueHandler::flush(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) {
-    
+
+    ASSERT(payload.size() > 0);
+
     size_t numArchived = 0;
-    
-    if (payload.size() > 0) {
-        MemoryStream s(payload);
-        s >> numArchived;
+    MemoryStream s(payload);
+    s >> numArchived;
+
+    if (numArchived == 0) {
+        return;
     }
 
     auto it = catalogues_.find(clientID);
     ASSERT(it != catalogues_.end());
 
-    it->second.locationsExpected = numArchived;
-    if (it->second.locationsArchived < numArchived) {
-        it->second.archivalCompleted.get();
+    {
+        std::lock_guard<std::mutex> lock(fieldLocationsMutex_);
+        it->second.locationsExpected = numArchived;     // setting locationsExpected also means that a flush has been requested
+        it->second.archivalCompleted = it->second.fieldLocationsReceived.get_future();
+        if (it->second.locationsArchived == numArchived) {
+            it->second.fieldLocationsReceived.set_value(numArchived);
+        }
     }
 
-    it->second.catalogue->flush();
+    it->second.archivalCompleted.wait();
+    {
+        std::lock_guard<std::mutex> lock(fieldLocationsMutex_);
+        it->second.fieldLocationsReceived = std::promise<size_t>{};
+        it->second.locationsExpected = 0;
+        it->second.locationsArchived = 0;
+    }
+
+    it->second.catalogue->flush(numArchived);
 
     Log::info() << "Flush complete" << std::endl;
     Log::status() << "Flush complete" << std::endl;
@@ -380,14 +416,12 @@ void CatalogueHandler::archiveBlob(const uint32_t clientID, const uint32_t reque
 
     MemoryStream s(data, length);
 
-    // fdb5::Key dbKey(s);
     fdb5::Key idxKey(s);
-    fdb5::InspectionKey key;
-    s >> key; // xxx no stream constructor for inspection key?
+    fdb5::Key datumKey(s);
 
     std::unique_ptr<FieldLocation> location(eckit::Reanimator<FieldLocation>::reanimate(s));
 
-    Log::debug<LibFdb5>() << "CatalogueHandler::archiveBlob key: " << idxKey << key << "  location: " << location->uri() << std::endl;
+    LOG_DEBUG_LIB(LibFdb5) << "CatalogueHandler::archiveBlob key: " << idxKey << datumKey << "  location: " << location->uri() << std::endl;
 
     std::map<uint32_t, CatalogueArchiver>::iterator it;
     {
@@ -401,17 +435,23 @@ void CatalogueHandler::archiveBlob(const uint32_t clientID, const uint32_t reque
     }
 
     it->second.catalogue->selectIndex(idxKey);
-    it->second.catalogue->archive(key, std::move(location));
-    it->second.locationsArchived++;
-    if (it->second.locationsExpected > 0 && it->second.locationsExpected == it->second.locationsArchived) {
-        it->second.fieldLocationsReceived.set_value(it->second.locationsExpected);
+    it->second.catalogue->archive(idxKey, datumKey, std::move(location));
+    {
+        std::lock_guard<std::mutex> lock(fieldLocationsMutex_);
+        it->second.locationsArchived++;
+        if (it->second.locationsExpected != 0 && it->second.archivalCompleted.valid() && it->second.locationsExpected == it->second.locationsArchived) {
+            it->second.fieldLocationsReceived.set_value(it->second.locationsExpected);
+        }
     }
 }
 
 bool CatalogueHandler::remove(bool control, uint32_t clientID) {
-    
+
     std::lock_guard<std::mutex> lock(handlerMutex_);
-    if (clientID == fdbId_) {
+
+    // is the client an FDB
+    auto it = fdbs_.find(clientID);
+    if (it != fdbs_.end()) {
         if (control) {
             fdbControlConnection_ = false;
             numControlConnection_--;
@@ -445,7 +485,6 @@ CatalogueWriter& CatalogueHandler::catalogue(uint32_t id, const Key& dbKey) {
         numDataConnection_++;
     }
     return *((catalogues_.emplace(id, CatalogueArchiver(!single_, dbKey, config_)).first)->second.catalogue);
-    // return *((catalogues_[id] = std::make_pair(single_ ? 1 : 2, CatalogueArchiver(dbKey, config_))).second.catalogue);
 }
 
 }  // namespace fdb5::remote
