@@ -10,8 +10,11 @@
 
 #pragma once
 
-#include "fdb5/remote/server/ServerConnection.h"
 #include "fdb5/api/FDB.h"
+#include "fdb5/database/Catalogue.h"
+#include "fdb5/remote/server/ServerConnection.h"
+
+#include <memory>
 
 namespace fdb5::remote {
 
@@ -20,19 +23,17 @@ namespace fdb5::remote {
 struct CatalogueArchiver {
     CatalogueArchiver(bool dataConnection, const Key& dbKey, const Config& config) :
         controlConnection(true), dataConnection(dataConnection),
-        catalogue(CatalogueWriterFactory::instance().build(dbKey, config)), locationsExpected(-1), locationsArchived(0) {
-        archivalCompleted = fieldLocationsReceived.get_future();
-    }
+        catalogue(CatalogueWriterFactory::instance().build(dbKey, config)), locationsExpected(0), locationsArchived(0) {}
 
     bool controlConnection;
     bool dataConnection;
-    
-    std::unique_ptr<CatalogueWriter> catalogue;
-    int32_t locationsExpected;
-    int32_t locationsArchived;
 
-    std::promise<int32_t> fieldLocationsReceived;
-    std::future<int32_t> archivalCompleted;
+    std::unique_ptr<CatalogueWriter> catalogue;
+    size_t locationsExpected;
+    size_t locationsArchived;
+
+    std::promise<size_t> fieldLocationsReceived;
+    std::future<size_t> archivalCompleted;
 };
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -40,14 +41,12 @@ class CatalogueHandler : public ServerConnection {
 public:  // methods
 
     CatalogueHandler(eckit::net::TCPSocket& socket, const Config& config);
-    ~CatalogueHandler();
+    ~CatalogueHandler() override;
 
 private:  // methods
 
     Handled handleControl(Message message, uint32_t clientID, uint32_t requestID) override;
     Handled handleControl(Message message, uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) override;
-    // Handled handleData(Message message, uint32_t clientID, uint32_t requestID) override;
-    // Handled handleData(Message message, uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) override;
 
     // API functionality
     template <typename HelperClass>
@@ -55,25 +54,28 @@ private:  // methods
 
     void flush(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload);
     void list(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload);
+    void axes(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload);
     void inspect(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload);
+    void stats(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload);
     void schema(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload);
     void stores(uint32_t clientID, uint32_t requestID);
+    void exists(uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload) const;
 
     void archiveBlob(const uint32_t clientID, const uint32_t requestID, const void* data, size_t length) override;
 
     bool remove(bool control, uint32_t clientID) override;
-    // bool handlers() override;
 
-    // CatalogueWriter& catalogue(uint32_t catalogueID);
     CatalogueWriter& catalogue(uint32_t catalogueID, const Key& dbKey);
 
 private:  // member
 
     // clientID --> <catalogue, locationsExpected, locationsArchived>
     std::map<uint32_t, CatalogueArchiver> catalogues_;
+    std::map<uint32_t, FDB> fdbs_;
+
+    std::mutex fdbMutex_;
+    std::mutex fieldLocationsMutex_;
 
-    FDB fdb_;
-    uint32_t fdbId_;
     bool fdbControlConnection_;
     bool fdbDataConnection_;
 };
