@@ -13,37 +13,50 @@
  * (Project ID: 671951) www.nextgenio.eu
  */
 
-#include <chrono>
+#include "fdb5/remote/server/ServerConnection.h"
 
 #include "eckit/config/Resource.h"
-#include "eckit/maths/Functions.h"
+#include "fdb5/LibFdb5.h"
+#include "fdb5/api/helpers/FDBToolRequest.h"
+#include "fdb5/fdb5_version.h"
+#include "fdb5/remote/Connection.h"
+#include "fdb5/remote/Messages.h"
+#include "fdb5/remote/server/AvailablePortList.h"
+
+#include "eckit/config/LocalConfiguration.h"
+#include "eckit/exception/Exceptions.h"
+#include "eckit/log/CodeLocation.h"
+#include "eckit/log/Log.h"
 #include "eckit/net/Endpoint.h"
-#include "eckit/runtime/Main.h"
+#include "eckit/net/TCPServer.h"
+#include "eckit/net/TCPSocket.h"
 #include "eckit/runtime/SessionID.h"
 #include "eckit/serialisation/MemoryStream.h"
-#include "eckit/log/Log.h"
 
-#include "fdb5/LibFdb5.h"
-#include "fdb5/fdb5_version.h"
-#include "fdb5/api/helpers/FDBToolRequest.h"
-#include "fdb5/database/Key.h"
-#include "fdb5/remote/AvailablePortList.h"
-#include "fdb5/remote/Messages.h"
-#include "fdb5/remote/RemoteFieldLocation.h"
-#include "fdb5/api/FDB.h"
-
-#include "fdb5/remote/server/ServerConnection.h"
+#include <algorithm>
+#include <cstddef>
+#include <cstdint>
+#include <exception>
+#include <future>
+#include <iterator>
+#include <map>
+#include <mutex>
+#include <ostream>
+#include <sstream>
+#include <string>
+#include <thread>
+#include <utility>
+#include <vector>
 
 namespace fdb5::remote {
 
+//----------------------------------------------------------------------------------------------------------------------
 // helpers
+
 namespace {
 
-class TCPException : public eckit::Exception {
-public:
-    TCPException(const std::string& msg, const eckit::CodeLocation& here) :
-        eckit::Exception(std::string("TCPException: ") + msg, here) {}
-};
+constexpr const auto defaultRetrieveQueueSize = 10000;
+constexpr const auto defaultArchiveQueueSize = 320;
 
 std::vector<int> intersection(const eckit::LocalConfiguration& c1, const eckit::LocalConfiguration& c2, const std::string& field){
 
@@ -62,33 +75,30 @@ std::vector<int> intersection(const eckit::LocalConfiguration& c1, const eckit::
 
 } // namespace
 
+//----------------------------------------------------------------------------------------------------------------------
 
-//size_t ServerConnection::queueSize_ = eckit::Resource<size_t>("fdbServerMaxQueueSize", 32);
+ServerConnection::ServerConnection(eckit::net::TCPSocket& socket, const Config& config)
+    : config_(config),
+      dataListenHostname_(config.getString("dataListenHostname", "")),
+      readLocationQueue_(eckit::Resource<size_t>("fdbRetrieveQueueSize", defaultRetrieveQueueSize)),
+      archiveQueue_(eckit::Resource<size_t>("fdbServerMaxQueueSize", defaultArchiveQueueSize)),
+      controlSocket_(socket) {
 
-ServerConnection::ServerConnection(eckit::net::TCPSocket& socket, const Config& config) :
-        Connection(), config_(config),
-        dataListenHostname_(config.getString("dataListenHostname", "")),
-        readLocationQueue_(eckit::Resource<size_t>("fdbRetrieveQueueSize", 10000)), 
-        archiveQueue_(eckit::Resource<size_t>("fdbServerMaxQueueSize", 32)),
-        controlSocket_(socket), numControlConnection_(0), numDataConnection_(0),
-        dataSocket_(nullptr), dataListener_(0) {
-
-    eckit::Log::debug<LibFdb5>() << "ServerConnection::ServerConnection initialized" << std::endl;
+    LOG_DEBUG_LIB(LibFdb5) << "ServerConnection::ServerConnection initialized" << std::endl;
 }
 
 ServerConnection::~ServerConnection() {
     // We don't want to die before the worker threads are cleaned up
     waitForWorkers();
 
-    std::this_thread::sleep_for(std::chrono::milliseconds(500));
+    if (archiveFuture_.valid()) {
+        archiveFuture_.wait();
+    }
 
-    // And notify the client that we are done.
-//     eckit::Log::info() << "Sending exit message to client" << std::endl;
-// //    write(Message::Exit, true, 0, 0);
-//     write(Message::Exit, false, 0, 0);
     eckit::Log::info() << "Done" << std::endl;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
 
 Handled ServerConnection::handleData(Message message, uint32_t clientID, uint32_t requestID) {
     try {
@@ -121,7 +131,7 @@ Handled ServerConnection::handleData(Message message, uint32_t clientID, uint32_
             case Message::Blob:
             case Message::MultiBlob:
                 queue(message, clientID, requestID, std::move(payload));
-                return Handled::Yes;
+                return Handled::Replied;
 
             default: {
                 std::stringstream ss;
@@ -145,9 +155,19 @@ Handled ServerConnection::handleData(Message message, uint32_t clientID, uint32_
 eckit::LocalConfiguration ServerConnection::availableFunctionality() const {
     eckit::LocalConfiguration conf;
 //    Add to the configuration all the components that require to be versioned, as in the following example, with a vector of supported version numbers
-    std::vector<int> remoteFieldLocationVersions = {1};
+    static std::vector<int> remoteFieldLocationVersions = {1};
+    static std::vector<int> numberOfConnections = config_.getIntVector("supportedConnections", {1,2});
+
+    ASSERT(0 < numberOfConnections.size());
+    ASSERT(numberOfConnections[0] == 1 || numberOfConnections[0] == 2);
+
+    ASSERT(numberOfConnections.size() <= 2);
+    if (numberOfConnections.size() > 1) {
+        ASSERT(numberOfConnections[0] == 1);
+        ASSERT(numberOfConnections[1] == 2);
+    }
+
     conf.set("RemoteFieldLocation", remoteFieldLocationVersions);
-    std::vector<int> numberOfConnections = {2};
     conf.set("NumberOfConnections", numberOfConnections);
     return conf;
 }
@@ -187,10 +207,9 @@ void ServerConnection::initialiseConnections() {
         agreedConf_ = eckit::LocalConfiguration();
         bool compatibleProtocol = true;
 
-        
         std::vector<int> rflCommon = intersection(clientAvailableFunctionality, serverConf, "RemoteFieldLocation");
         if (rflCommon.size() > 0) {
-            eckit::Log::debug<LibFdb5>() << "Protocol negotiation - RemoteFieldLocation version " << rflCommon.back() << std::endl;
+            LOG_DEBUG_LIB(LibFdb5) << "Protocol negotiation - RemoteFieldLocation version " << rflCommon.back() << std::endl;
             agreedConf_.set("RemoteFieldLocation", rflCommon.back());
         } else {
             compatibleProtocol = false;
@@ -216,7 +235,7 @@ void ServerConnection::initialiseConnections() {
                 }
             }
 
-            eckit::Log::debug<LibFdb5>() << "Protocol negotiation - NumberOfConnections " << ncSelected << std::endl;
+            LOG_DEBUG_LIB(LibFdb5) << "Protocol negotiation - NumberOfConnections " << ncSelected << std::endl;
             agreedConf_.set("NumberOfConnections", ncSelected);
             single_ = (ncSelected == 1);
         }
@@ -235,38 +254,35 @@ void ServerConnection::initialiseConnections() {
     //               server has multiple, then we use that on, whilst retaining
     //               the capacity in the protocol for the server to make a choice.
 
+    std::future<bool> dataSocketFuture;
     eckit::net::Endpoint dataEndpoint;
     if (single_) {
         dataEndpoint = endpointFromClient;
     } else {
+        std::lock_guard<std::mutex> lock(dataPortMutex_);
         dataSocket_.reset(new eckit::net::EphemeralTCPServer(selectDataPort()));
+        ASSERT(dataSocket_->socket() != -1);
+
         dataEndpoint = eckit::net::Endpoint{endpointFromClient.hostname(), dataSocket_->localPort()};
+
+        dataSocketFuture = std::async(std::launch::async, [this] { dataSocket_->accept(); return true; });
     }
 
-    eckit::Log::info() << "Sending data endpoint to client: " << dataEndpoint << std::endl;
-    {
-        eckit::Buffer startupBuffer(1024);
-        eckit::MemoryStream s(startupBuffer);
+    eckit::Buffer startupBuffer(1024);
+    eckit::MemoryStream s(startupBuffer);
 
-        s << clientSession;
-        s << sessionID_;
-        s << dataEndpoint;
-        s << agreedConf_.get();
-        // s << storeEndpoint; // xxx single-store case only: we cant do this with multiple stores // For now, dont send the store endpoint to the client 
+    s << clientSession;
+    s << sessionID_;
+    s << dataEndpoint;
+    s << agreedConf_.get();
 
-        eckit::Log::debug<LibFdb5>() << "Protocol negotiation - configuration: " << agreedConf_ <<std::endl;
+    LOG_DEBUG_LIB(LibFdb5) << "Protocol negotiation - configuration: " << agreedConf_ <<std::endl;
 
-        write(Message::Startup, true, 0, 0, std::vector<std::pair<const void*, uint32_t>>{{startupBuffer.data(), s.position()}});
-    }
-
-
-    if (!errorMsg.empty()) {
-        error(errorMsg, hdr.clientID(), hdr.requestID);
-        return;
-    }
+    write(Message::Startup, true, 0, 0, startupBuffer.data(), s.position());
 
     if (!single_) {
-        dataSocket_->accept();
+        ASSERT(dataSocketFuture.valid());
+        dataSocketFuture.wait();
 
         // Check the response from the client.
         // Ensure that the hostname matches the original hostname, and that
@@ -276,7 +292,7 @@ void ServerConnection::initialiseConnections() {
         MessageHeader dataHdr;
         eckit::Buffer payload2 = readData(dataHdr);
 
-        ASSERT(dataHdr.version == CurrentVersion);
+        ASSERT(dataHdr.version == MessageHeader::currentVersion);
         ASSERT(dataHdr.message == Message::Startup);
         ASSERT(dataHdr.requestID == 0);
 
@@ -296,6 +312,11 @@ void ServerConnection::initialiseConnections() {
             throw eckit::BadValue(ss.str(), Here());
         }
     }
+
+    if (!errorMsg.empty()) {
+        error(errorMsg, hdr.clientID(), hdr.requestID);
+        return;
+    }
 }
 
 int ServerConnection::selectDataPort() {
@@ -334,9 +355,9 @@ size_t ServerConnection::archiveThreadLoop() {
                     const void* payloadData = charData;
                     charData += hdr->payloadSize;
 
-                    const decltype(EndMarker)* e = static_cast<const decltype(EndMarker)*>(static_cast<const void*>(charData));
-                    ASSERT(*e == EndMarker);
-                    charData += sizeof(EndMarker);
+                    const auto* e = static_cast<const MessageHeader::MarkerType*>(static_cast<const void*>(charData));
+                    ASSERT(*e == MessageHeader::EndMarker);
+                    charData += MessageHeader::markerBytes;
 
                     archiveBlob(elem.clientID_, elem.requestID_, payloadData, hdr->payloadSize);
                     totalArchived += 1;
@@ -348,12 +369,6 @@ size_t ServerConnection::archiveThreadLoop() {
                 totalArchived += 1;
             }
         }
-
-        // eckit::Buffer buffer(1024);
-        // eckit::MemoryStream stream(buffer);
-        // stream << totalArchived;
-        
-        // write(Message::Complete, false, 0, 0, buffer, stream.position());
     }
     catch (...) {
         // Ensure exception propagates across the queue back to the parent thread.
@@ -364,12 +379,9 @@ size_t ServerConnection::archiveThreadLoop() {
     return totalArchived;
 }
 
-
-
 void ServerConnection::listeningThreadLoopData() {
 
     MessageHeader hdr;
-    uint32_t archiverID;
 
     try {
 
@@ -380,15 +392,14 @@ void ServerConnection::listeningThreadLoopData() {
             eckit::Buffer payload = readData(hdr); // READ DATA
 
             if (hdr.message == Message::Exit) {
-                if (remove(false, hdr.clientID())) {
+                ASSERT(hdr.clientID() == 0);
 
-                    eckit::Log::status() << "Terminating DATA listener" << std::endl;
-                    eckit::Log::info() << "Terminating DATA listener" << std::endl;
+                eckit::Log::status() << "Terminating DATA listener" << std::endl;
+                eckit::Log::info() << "Terminating DATA listener" << std::endl;
 
-                    break;
-                }
+                break;
             } else {
-                
+
                 Handled handled;
                 if (payload.size() == 0) {
                     handled = handleData(hdr.message, hdr.clientID(), hdr.requestID);
@@ -396,26 +407,18 @@ void ServerConnection::listeningThreadLoopData() {
                     handled = handleData(hdr.message, hdr.clientID(), hdr.requestID, std::move(payload));
                 }
 
-
                 switch (handled)
                 {
-                    case Handled::Replied: // nothing to do
-                        break;
                     case Handled::YesRemoveArchiveListener:
-                        dataListener_--;
-                        if (dataListener_ == 0) {
-                            // return;
-                            // listeningThreadData.join();
-                        }
-                        break;
                     case Handled::YesRemoveReadListener:
-                        dataListener_--;
-                        if (dataListener_ == 0) {
-                            // return;
+                        {
+                            std::lock_guard<std::mutex> lock(handlerMutex_);
+                            numDataListener_--;
+                            if (numDataListener_ == 0) { return; }
+                            break;
                         }
-                        break;
+                    case Handled::Replied: // nothing to do
                     case Handled::Yes:
-        //                write(Message::Received, false, hdr.clientID(), hdr.requestID);
                         break;
                     case Handled::No:
                     default:
@@ -426,129 +429,84 @@ void ServerConnection::listeningThreadLoopData() {
             }
         }
 
-        // // Trigger cleanup of the workers
-        // auto q = archiveQueues_.find(archiverID);
-        // ASSERT(q != archiveQueues_.end());
-        // q->second.close();
-
-        // auto w = archiveFuture_.find(archiverID);
-        // ASSERT(w != archiveFuture_.end());
-        // // Ensure worker is done
-        // ASSERT(w->second.valid());
-        // totalArchived = worker.get();  // n.b. use of async, get() propagates any exceptions.
     }
     catch (std::exception& e) {
         // n.b. more general than eckit::Exception
         error(e.what(), hdr.clientID(), hdr.requestID);
-        // auto q = archiveQueues_.find(archiverID);
-        // if(q != archiveQueues_.end()) {
-        //     q->second.interrupt(std::current_exception());
-        // }
         throw;
     }
     catch (...) {
         error("Caught unexpected, unknown exception in retrieve worker", hdr.clientID(), hdr.requestID);
-        // auto q = archiveQueues_.find(archiverID);
-        // if(q != archiveQueues_.end()) {
-        //     q->second.interrupt(std::current_exception());
-        // }
         throw;
     }
 }
 
 void ServerConnection::handle() {
     initialiseConnections();
- 
+
     std::thread listeningThreadData;
-    // if (!single_) {
-    //     listeningThreadData = std::thread([this] { listeningThreadLoopData(); });
-    // }
 
     MessageHeader hdr;
 
     // listen loop
     while (true) {
-        //tidyWorkers();
-
         eckit::Buffer payload = readControl(hdr); // READ CONTROL
-        eckit::Log::debug<LibFdb5>() << "ServerConnection::handle - got [message=" << hdr.message << ",clientID="<< hdr.clientID() << ",requestID=" << hdr.requestID << ",payload=" << hdr.payloadSize << "]" << std::endl;
+        LOG_DEBUG_LIB(LibFdb5) << "ServerConnection::handle - got [message=" << hdr.message << ",clientID="<< hdr.clientID() << ",requestID=" << hdr.requestID << ",payload=" << hdr.payloadSize << "]" << std::endl;
 
-        if (hdr.message == Message::Exit) {
-            if (remove(true, hdr.clientID())) {
+        if (hdr.message == Message::Stop) {
+            ASSERT(hdr.clientID());
+            remove(true, hdr.clientID());
 
-                write(Message::Exit, false, hdr.clientID(), 0);
+        } else {
+            if (hdr.message == Message::Exit) {
+                ASSERT(hdr.clientID() == 0);
 
                 eckit::Log::status() << "Terminating CONTROL listener" << std::endl;
                 eckit::Log::info() << "Terminating CONTROL listener" << std::endl;
 
                 break;
             }
-        } else {
+            else {
 
-            Handled handled = Handled::No;
-            ASSERT(single_ || hdr.control());
+                Handled handled = Handled::No;
+                ASSERT(single_ || hdr.control());
 
-            if (payload.size()) {
-                if (hdr.control()) {
-                    handled = handleControl(hdr.message, hdr.clientID(), hdr.requestID, std::move(payload));
-                } else {
-                    handled = handleData(hdr.message, hdr.clientID(), hdr.requestID, std::move(payload));
-                }
-            } else {
-                if (hdr.control()) {
-                    handled = handleControl(hdr.message, hdr.clientID(), hdr.requestID);
-                } else {
-                    handled = handleData(hdr.message, hdr.clientID(), hdr.requestID);
-                }
-            }
-            
-
-            switch (handled)
-            {
-                case Handled::Replied: // nothing to do
-                    break;
-                // case Handled::YesRemoveArchiveListener:
-                //     dataListener_--;
-                //     if (dataListener_ == 0) {
-                //         //return;
-                //         // listeningThreadData.join();
-                //     }
-                //     break;
-                case Handled::YesAddArchiveListener:
-                    {
-                        std::lock_guard<std::mutex> lock(handlerMutex_);
-                        dataListener_++;
-                        if (dataListener_ == 1) {
-                            listeningThreadData = std::thread([this] { listeningThreadLoopData(); });
-                        }
+                if (payload.size()) {
+                    if (hdr.control()) {
+                        handled = handleControl(hdr.message, hdr.clientID(), hdr.requestID, std::move(payload));
+                    } else {
+                        handled = handleData(hdr.message, hdr.clientID(), hdr.requestID, std::move(payload));
                     }
-                    write(Message::Received, false, hdr.clientID(), hdr.requestID);
-                    break;
-                // case Handled::YesRemoveReadListener:
-                //     dataListener_--;
-                //     if (dataListener_ == 0) {
-                //         //return;
-                //         // listeningThreadData.join();
-                //     }
-                //     break;
-                case Handled::YesAddReadListener:
-                    {
-                        std::lock_guard<std::mutex> lock(handlerMutex_);
-                        dataListener_++;
-                        if (dataListener_ == 1) {
-                            listeningThreadData = std::thread([this] { listeningThreadLoopData(); });
-                        }
+                } else {
+                    if (hdr.control()) {
+                        handled = handleControl(hdr.message, hdr.clientID(), hdr.requestID);
+                    } else {
+                        handled = handleData(hdr.message, hdr.clientID(), hdr.requestID);
                     }
-                    write(Message::Received, false, hdr.clientID(), hdr.requestID);
-                    break;
-                case Handled::Yes:
-                    write(Message::Received, false, hdr.clientID(), hdr.requestID);
-                    break;
-                case Handled::No:
-                default:
-                    std::stringstream ss;
-                    ss << "Unable to handle message " << hdr.message;
-                    error(ss.str(), hdr.clientID(), hdr.requestID);
+                }
+
+                switch (handled)
+                {
+                    case Handled::Replied: // nothing to do
+                        break;
+                    case Handled::YesAddArchiveListener:
+                    case Handled::YesAddReadListener:
+                        {
+                            std::lock_guard<std::mutex> lock(handlerMutex_);
+                            numDataListener_++;
+                            if (numDataListener_ == 1 && !single_) {
+                                listeningThreadData = std::thread([this] { listeningThreadLoopData(); });
+                            }
+                        }
+                    case Handled::Yes:
+                        write(Message::Received, true, hdr.clientID(), hdr.requestID);
+                        break;
+                    case Handled::No:
+                    default:
+                        std::stringstream ss;
+                        ss << "Unable to handle message " << hdr.message;
+                        error(ss.str(), hdr.clientID(), hdr.requestID);
+                }
             }
         }
     }
@@ -556,6 +514,10 @@ void ServerConnection::handle() {
     if (listeningThreadData.joinable()) {
         listeningThreadData.join();
     }
+    ASSERT(archiveQueue_.empty());
+    archiveQueue_.close();
+
+    teardown();
 }
 
 void ServerConnection::handleException(std::exception_ptr e) {
@@ -592,7 +554,6 @@ void ServerConnection::tidyWorkers() {
     }
 }
 
-
 void ServerConnection::archiver() {
 
     // Ensure that we aren't already running a catalogue/store
@@ -600,11 +561,6 @@ void ServerConnection::archiver() {
         // Start archive worker thread
         archiveFuture_ = std::async(std::launch::async, [this] { return archiveThreadLoop(); });
     }
-
-    // // Start data reader thread if double connection and we aren't already running it
-    // if (!single_ &&  !dataReader_.valid()) {
-    //     dataReader_ = std::async(std::launch::async, [this] { return listeningThreadLoopData(); });
-    // }
 }
 
 void ServerConnection::waitForWorkers() {
@@ -619,6 +575,7 @@ void ServerConnection::waitForWorkers() {
         eckit::Log::error() << "Thread complete" << std::endl;
     }
 
+    std::lock_guard<std::mutex> lock(readLocationMutex_);
     if (readLocationWorker_.joinable()) {
         readLocationWorker_.join();
     }
