@@ -19,8 +19,8 @@
 
 #include <future>
 #include <mutex>
-#include <unordered_map>
 
+#include "eckit/container/Queue.h"
 #include "eckit/io/Buffer.h"
 #include "eckit/io/DataHandle.h"
 #include "eckit/net/TCPServer.h"
@@ -66,7 +66,7 @@ struct readLocationElem {
     std::unique_ptr<eckit::DataHandle> readLocation;
 
     readLocationElem() : clientID(0), requestID(0), readLocation(nullptr) {}
-    
+
     readLocationElem(uint32_t clientID, uint32_t requestID, std::unique_ptr<eckit::DataHandle> readLocation) :
         clientID(clientID), requestID(requestID), readLocation(std::move(readLocation)) {}
 };
@@ -87,7 +87,7 @@ struct ArchiveElem {
 class ServerConnection : public Connection, public Handler {
 public:  // methods
     ServerConnection(eckit::net::TCPSocket& socket, const Config& config);
-    ~ServerConnection();
+    ~ServerConnection() override;
 
     void initialiseConnections();
 
@@ -102,42 +102,32 @@ public:  // methods
 
 protected:
 
-    // Handler& handler(uint32_t id);
-    // virtual Handler& handler(uint32_t id, Buffer& payload) = 0;
-
     // socket methods
     int selectDataPort();
-//    virtual void initialiseConnections();
     eckit::LocalConfiguration availableFunctionality() const;
-    
-        // Worker functionality
+
+    // Worker functionality
     void tidyWorkers();
     void waitForWorkers();
 
     // archival thread
     size_t archiveThreadLoop();
-    virtual void archiveBlob(const uint32_t clientID, const uint32_t requestID, const void* data, size_t length) = 0;
+
+    virtual void archiveBlob(uint32_t clientID, uint32_t requestID, const void* data, size_t length) = 0;
 
     // archival helper methods
     void archiver();
-    // emplace new ArchiveElem to archival queue
     void queue(Message message, uint32_t clientID, uint32_t requestID, eckit::Buffer&& payload);
-    // // retrieve archival queue
-    // eckit::Queue<ArchiveElem>& queue();
 
     void handleException(std::exception_ptr e) override;
 
-    // void archive(const MessageHeader& hdr);
-    // virtual size_t archiveThreadLoop() = 0;
-
 private:
 
-    // void controlWrite(Message msg, uint32_t clientID, uint32_t requestID, const void* payload = nullptr, uint32_t payloadLength = 0);
-    // void controlWrite(const void* data, size_t length);
     void listeningThreadLoopData();
 
-    eckit::net::TCPSocket& controlSocket() override { return controlSocket_; }
-    eckit::net::TCPSocket& dataSocket() override { 
+    const eckit::net::TCPSocket& controlSocket() const override { return controlSocket_; }
+
+    const eckit::net::TCPSocket& dataSocket() const override {
         ASSERT(dataSocket_);
         return *dataSocket_;
     }
@@ -145,9 +135,7 @@ private:
 protected:
 
     virtual bool remove(bool control, uint32_t clientID) = 0;
-    // virtual bool handlers() = 0;
 
-    // std::map<uint32_t, Handler*> handlers_;
     Config config_;
     std::string dataListenHostname_;
 
@@ -155,10 +143,9 @@ protected:
 
     eckit::SessionID sessionID_;
     eckit::LocalConfiguration agreedConf_;
-    // std::mutex controlWriteMutex_;
-    // std::mutex dataWriteMutex_;
+    std::mutex readLocationMutex_;
     std::thread readLocationWorker_;
-    
+
     std::map<uint32_t, std::future<void>> workerThreads_;
     eckit::Queue<ArchiveElem> archiveQueue_;
     std::future<size_t> archiveFuture_;
@@ -166,14 +153,17 @@ protected:
     eckit::net::TCPSocket controlSocket_;
 
     std::mutex handlerMutex_;
-    size_t numControlConnection_;
-    size_t numDataConnection_;
+    size_t     numControlConnection_ {0};
+    size_t     numDataConnection_ {0};
 
 private:
 
+    std::mutex dataPortMutex_;
+
     // data connection
     std::unique_ptr<eckit::net::EphemeralTCPServer> dataSocket_;
-    size_t dataListener_;
+
+    size_t numDataListener_ {0};
 };
 
 //----------------------------------------------------------------------------------------------------------------------
