@@ -8,22 +8,32 @@
  * does it submit to any jurisdiction.
  */
 
-#include "eckit/config/Resource.h"
-#include "eckit/serialisation/MemoryStream.h"
-
-#include "fdb5/LibFdb5.h"
-#include "fdb5/database/Store.h"
 #include "fdb5/remote/server/StoreHandler.h"
 
+#include "fdb5/LibFdb5.h"
+#include "fdb5/database/Key.h"
+#include "fdb5/database/Store.h"
+#include "fdb5/remote/Messages.h"
+#include "fdb5/remote/server/ServerConnection.h"
+
+#include "eckit/net/TCPSocket.h"
+#include "eckit/serialisation/MemoryStream.h"
+
+#include <cstdint>
+#include <memory>
+#include <mutex>
+#include <utility>
+
 using namespace eckit;
-using metkit::mars::MarsRequest;
 
 namespace fdb5::remote {
 
-StoreHandler::StoreHandler(eckit::net::TCPSocket& socket, const Config& config):
-    ServerConnection(socket, config) {}
+//----------------------------------------------------------------------------------------------------------------------
 
-StoreHandler::~StoreHandler() {}
+StoreHandler::StoreHandler(eckit::net::TCPSocket& socket, const Config& config):
+    ServerConnection(socket, config) {
+        LibFdb5::instance().constructorCallback()(*this);
+    }
 
 Handled StoreHandler::handleControl(Message message, uint32_t clientID, uint32_t requestID) {
 
@@ -32,10 +42,6 @@ Handled StoreHandler::handleControl(Message message, uint32_t clientID, uint32_t
             case Message::Store: // notification that the client is starting to send data for archival
                 archiver();
                 return Handled::YesAddArchiveListener;
-            
-            // case Message::Flush: // notification that the client has sent all data for archival
-            //     flush(clientID, requestID, eckit::Buffer{0});
-            //     return Handled::YesRemoveArchiveListener; // ????
 
             default: {
                 std::stringstream ss;
@@ -69,6 +75,10 @@ Handled StoreHandler::handleControl(Message message, uint32_t clientID, uint32_t
                 flush(clientID, requestID, payload);
                 return Handled::Yes;
 
+            case Message::Exists:  // given key (payload), check if store exists
+                exists(clientID, requestID, payload);
+                return Handled::Replied;
+
             default: {
                 std::stringstream ss;
                 ss << "ERROR: Unexpected message recieved (" << message << "). ABORTING";
@@ -88,17 +98,22 @@ Handled StoreHandler::handleControl(Message message, uint32_t clientID, uint32_t
     return Handled::No;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void StoreHandler::read(uint32_t clientID, uint32_t requestID, const eckit::Buffer& payload) {
 
-    if (!readLocationWorker_.joinable()) {
-        readLocationWorker_ = std::thread([this] { readLocationThreadLoop(); });
+    {
+        std::lock_guard<std::mutex> lock(readLocationMutex_);
+        if (!readLocationWorker_.joinable()) {
+            readLocationWorker_ = std::thread([this] { readLocationThreadLoop(); });
+        }
     }
 
     MemoryStream s(payload);
 
     std::unique_ptr<FieldLocation> location(eckit::Reanimator<FieldLocation>::reanimate(s));
 
-    Log::debug<LibFdb5>() << "Queuing for read: " << requestID << " " << *location << std::endl;
+    LOG_DEBUG_LIB(LibFdb5) << "Queuing for read: " << requestID << " " << *location << std::endl;
 
     std::unique_ptr<eckit::DataHandle> dh;
     dh.reset(location->dataHandle());
@@ -123,26 +138,26 @@ void StoreHandler::writeToParent(const uint32_t clientID, const uint32_t request
         Log::status() << "Reading: " << requestID << std::endl;
         // Write the data to the parent, in chunks if necessary.
 
-        Buffer writeBuffer(10 * 1024 * 1024);
+        Buffer writeBuffer(4 * 1024 * 1024 - 2048); // slightly smaller than 4MiB to nicely fit in a TCP window with scale factor 6
         long dataRead;
 
         dh->openForRead();
-        Log::debug<LibFdb5>() << "Reading: " << requestID << " dh size: " << dh->size()
+        LOG_DEBUG_LIB(LibFdb5) << "Reading: " << requestID << " dh size: " << dh->size()
                               << std::endl;
 
         while ((dataRead = dh->read(writeBuffer, writeBuffer.size())) != 0) {
-            write(Message::Blob, false, clientID, requestID, std::vector<std::pair<const void*, uint32_t>>{{writeBuffer, dataRead}});
+            write(Message::Blob, false, clientID, requestID, writeBuffer, dataRead);
         }
 
         // And when we are done, add a complete message.
 
-        Log::debug<LibFdb5>() << "Writing retrieve complete message: " << requestID
+        LOG_DEBUG_LIB(LibFdb5) << "Writing retrieve complete message: " << requestID
                               << std::endl;
 
         write(Message::Complete, false, clientID, requestID);
 
         Log::status() << "Done retrieve: " << requestID << std::endl;
-        Log::debug<LibFdb5>() << "Done retrieve: " << requestID << std::endl;
+        LOG_DEBUG_LIB(LibFdb5) << "Done retrieve: " << requestID << std::endl;
     }
     catch (std::exception& e) {
         // n.b. more general than eckit::Exception
@@ -156,7 +171,7 @@ void StoreHandler::writeToParent(const uint32_t clientID, const uint32_t request
 
 
 void StoreHandler::archiveBlob(const uint32_t clientID, const uint32_t requestID, const void* data, size_t length) {
-    
+
     MemoryStream s(data, length);
 
     fdb5::Key dbKey(s);
@@ -167,19 +182,26 @@ void StoreHandler::archiveBlob(const uint32_t clientID, const uint32_t requestID
 
     const char* charData = static_cast<const char*>(data);  // To allow pointer arithmetic
     Log::status() << "Archiving data: " << ss_key.str() << std::endl;
-    
+
     Store& ss = store(clientID, dbKey);
 
-    auto futureLocation = ss.archive(idxKey, charData + s.position(), length - s.position());
+    std::shared_ptr<const FieldLocation> location = ss.archive(idxKey, charData + s.position(), length - s.position());
+
+    std::promise<std::shared_ptr<const FieldLocation>> promise;
+    promise.set_value(location);
+
+    eckit::StringDict dict = dbKey.keyDict();
+    dict.insert(idxKey.keyDict().begin(), idxKey.keyDict().end());
+    const Key fullkey(dict); /// @note: we do not have the third level of the key.
+
+    archiveCallback_(fullkey, charData + s.position(), length - s.position(), promise.get_future());
+
     Log::status() << "Archiving done: " << ss_key.str() << std::endl;
-    
-    auto loc = futureLocation.get();
 
     eckit::Buffer buffer(16 * 1024);
     MemoryStream stream(buffer);
-//    stream << archiverID;
-    stream << (*loc);
-    Connection::write(Message::Store, false, clientID, requestID, buffer, stream.position());
+    stream << (*location);
+    Connection::write(Message::Store, true, clientID, requestID, buffer, stream.position());
 }
 
 void StoreHandler::flush(uint32_t clientID, uint32_t requestID, const eckit::Buffer& payload) {
@@ -193,34 +215,24 @@ void StoreHandler::flush(uint32_t clientID, uint32_t requestID, const eckit::Buf
         stream >> numArchived;
     }
 
-    ASSERT(numArchived == 0 || archiveFuture_.valid());
+    if (numArchived > 0) {
+        ASSERT(archiveFuture_.valid());
 
-    auto it = stores_.find(clientID);
-    ASSERT(it != stores_.end());
-    it->second.store->flush();
+        std::lock_guard<std::mutex> lock(handlerMutex_);
+        auto it = stores_.find(clientID);
+        ASSERT(it != stores_.end());
+        it->second.store->flush();
 
-    // if (archiveFuture_.valid()) {
-    //     // Ensure that the expected number of fields have been written, and that the
-    //     // archive worker processes have been cleanly wound up.
-    //     size_t n = archiveFuture_.get();
-    //     ASSERT(numArchived == n);
-
-    //     // Do the actual flush!
-    //     Log::info() << "Flushing" << std::endl;
-    //     Log::status() << "Flushing" << std::endl;
-    //     // for (auto it = stores_.begin(); it != stores_.end(); it++) {
-    //     //     it->second->flush();
-    //     // }
-    // }
+        flushCallback_();
+    }
 
     Log::info() << "Flush complete" << std::endl;
     Log::status() << "Flush complete" << std::endl;
 }
 
 bool StoreHandler::remove(bool control, uint32_t clientID) {
-    
-    std::lock_guard<std::mutex> lock(handlerMutex_);
 
+    std::lock_guard<std::mutex> lock(handlerMutex_);
     auto it = stores_.find(clientID);
     if (it != stores_.end()) {
         if (control) {
@@ -237,18 +249,14 @@ bool StoreHandler::remove(bool control, uint32_t clientID) {
     return ((control ? numControlConnection_ : numDataConnection_) == 0);
 }
 
-// bool StoreHandler::handlers() {
-//     std::lock_guard<std::mutex> lock(handlerMutex_);
-//     return stores_.empty();
-// }
-
 Store& StoreHandler::store(uint32_t clientID) {
 
     std::lock_guard<std::mutex> lock(handlerMutex_);
     auto it = stores_.find(clientID);
     if (it == stores_.end()) {
         std::string what("Requested Store has not been loaded id: " + std::to_string(clientID));
-        write(Message::Error, false, 0, 0, what.c_str(), what.length());
+        Log::error() << what << std::endl;
+        write(Message::Error, true, 0, 0, what.c_str(), what.length());
         throw;
     }
 
@@ -270,4 +278,25 @@ Store& StoreHandler::store(uint32_t clientID, const Key& dbKey) {
     return *((stores_.emplace(clientID, StoreHelper(!single_, dbKey, config_)).first)->second.store);
 }
 
+void StoreHandler::exists(const uint32_t clientID, const uint32_t requestID, const eckit::Buffer& payload) const {
+
+    ASSERT(payload.size() > 0);
+
+    bool exists = false;
+
+    {
+        eckit::MemoryStream stream(payload);
+        const Key           dbKey(stream);
+        exists = StoreFactory::instance().build(dbKey, config_)->exists();
+    }
+
+    eckit::Buffer       existBuf(5);
+    eckit::MemoryStream stream(existBuf);
+    stream << exists;
+
+    write(Message::Received, true, clientID, requestID, existBuf.data(), stream.position());
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 }  // namespace fdb5::remote
