@@ -8,7 +8,7 @@
  * does it submit to any jurisdiction.
  */
 
-#include "eckit/log/Log.h"
+#include <ostream>
 
 #include "fdb5/rules/MatchAlways.h"
 #include "fdb5/types/TypesRegistry.h"
@@ -22,10 +22,6 @@ eckit::ClassSpec MatchAlways::classSpec_ = { &Matcher::classSpec(), "MatchAlways
 eckit::Reanimator<MatchAlways> MatchAlways::reanimator_;
 
 
-MatchAlways::MatchAlways() :
-    Matcher() {
-}
-
 MatchAlways::MatchAlways(eckit::Stream&) :
     Matcher() {
 }
@@ -33,13 +29,6 @@ MatchAlways::MatchAlways(eckit::Stream&) :
 void MatchAlways::encode(eckit::Stream& s) const {
 }
 
-MatchAlways::~MatchAlways() {
-}
-
-bool MatchAlways::match(const std::string&, const Key&) const {
-    return true;
-}
-
 void MatchAlways::dump(std::ostream &s, const std::string &keyword, const TypesRegistry &registry) const {
     registry.dump(s, keyword);
 }
