@@ -16,6 +16,7 @@
 #pragma once
 
 #include <iosfwd>
+#include <string>
 
 #include "fdb5/rules/Matcher.h"
 
@@ -26,24 +27,23 @@ namespace fdb5 {
 class MatchAlways : public Matcher {
 
 public: // methods
+    MatchAlways() = default;
 
-    MatchAlways();
     MatchAlways(eckit::Stream& s);
+    bool match(const std::string& /*keyword*/, const Key& /*key*/) const override { return true; }
 
-    virtual ~MatchAlways() override;
+    bool match(const std::string& /*value*/) const override { return true; }
 
-    virtual bool match(const std::string &keyword, const Key &key) const override;
+    void dump(std::ostream& s, const std::string& keyword, const TypesRegistry& registry) const override;
 
-    virtual void dump(std::ostream &s, const std::string &keyword, const TypesRegistry &registry) const override;
-
-	const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
-	static const eckit::ClassSpec&  classSpec() { return classSpec_; }
+    const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
+    static const eckit::ClassSpec&  classSpec() { return classSpec_; }
 
 private: // methods
 
     void encode(eckit::Stream&) const override;
 
-    virtual void print( std::ostream &out ) const override;
+    void print( std::ostream& out ) const override;
 
 private: // members
 
