@@ -8,6 +8,8 @@
  * does it submit to any jurisdiction.
  */
 
+#include <ostream>
+
 #include "fdb5/database/Key.h"
 #include "fdb5/rules/MatchAny.h"
 #include "fdb5/types/TypesRegistry.h"
@@ -21,7 +23,7 @@ eckit::ClassSpec MatchAny::classSpec_ = { &Matcher::classSpec(), "MatchAny", };
 eckit::Reanimator<MatchAny> MatchAny::reanimator_;
 
 
-MatchAny::MatchAny(const std::set<std::string> &values) :
+MatchAny::MatchAny(const std::set<std::string>& values) :
     Matcher(),
     values_(values) {
 }
@@ -46,18 +48,15 @@ void MatchAny::encode(eckit::Stream& s) const {
     }
 }
 
-MatchAny::~MatchAny() {
+bool MatchAny::match(const std::string &keyword, const Key& key) const {
+
+    if (const auto [iter, found] = key.find(keyword); found) { return match(iter->second); }
+
+    return false;
 }
 
-bool MatchAny::match(const std::string &keyword, const Key &key) const {
-
-    Key::const_iterator i = key.find(keyword);
-
-    if (i == key.end()) {
-        return false;
-    }
-
-    return (values_.find(i->second) != values_.end());
+bool MatchAny::match(const std::string& value) const {
+    return (values_.find(value) != values_.end());
 }
 
 void MatchAny::dump(std::ostream &s, const std::string &keyword, const TypesRegistry &registry) const {
