@@ -18,6 +18,7 @@
 
 #include <iosfwd>
 #include <string>
+#include <vector>
 
 #include "fdb5/rules/Matcher.h"
 
@@ -28,28 +29,29 @@ namespace fdb5 {
 class MatchHidden : public Matcher{
 
 public: // methods
+    MatchHidden(std::string def);
 
-    MatchHidden(const std::string &def);
     MatchHidden(eckit::Stream& s);
 
-    virtual ~MatchHidden() override;
+    bool match(const std::string& /*value*/) const override { return true; }
 
-    virtual bool match(const std::string &keyword, const Key &key) const override;
+    bool match(const std::string& /*keyword*/, const Key& /*key*/) const override { return true; }
 
-    virtual void dump(std::ostream &s, const std::string &keyword, const TypesRegistry &registry) const override;
+    void dump(std::ostream& s, const std::string& keyword, const TypesRegistry& registry) const override;
 
-	const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
-	static const eckit::ClassSpec&  classSpec() { return classSpec_; }
+    const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
+    static const eckit::ClassSpec&  classSpec() { return classSpec_; }
 
 private: // methods
+    void encode(eckit::Stream& stream) const override;
 
-    void encode(eckit::Stream&) const override;
+    bool optional() const override { return true; }
 
-    virtual bool optional() const override;
-    virtual const std::string &value(const Key &, const std::string &keyword) const override;
-    virtual const std::vector<std::string>& values(const metkit::mars::MarsRequest& rq, const std::string& keyword) const override;
-    virtual void print( std::ostream &out ) const override;
-    virtual const std::string &defaultValue() const override;
+    const std::string &value(const Key&, const std::string& keyword) const override;
+    const std::vector<std::string>& values(const metkit::mars::MarsRequest& rq, const std::string& keyword) const override;
+    const std::string& defaultValue() const override;
+
+    void print(std::ostream& out) const override;
 
 private: // members
 
