@@ -25,32 +25,32 @@ namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-class MatchOptional : public Matcher{
+class MatchOptional : public Matcher {
 
 public: // methods
+    MatchOptional(std::string def);
 
-    MatchOptional(const std::string &def);
     MatchOptional(eckit::Stream& s);
 
-    virtual ~MatchOptional() override;
+    bool match(const std::string& value) const override;
 
-    virtual bool match(const std::string &keyword, const Key &key) const override;
+    bool match(const std::string& keyword, const Key& key) const override;
 
-    virtual void dump(std::ostream &s, const std::string &keyword, const TypesRegistry &registry) const override;
+    void dump(std::ostream& s, const std::string& keyword, const TypesRegistry& registry) const override;
 
-	const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
-	static const eckit::ClassSpec&  classSpec() { return classSpec_; }
+    const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
+    static const eckit::ClassSpec&  classSpec() { return classSpec_; }
 
 private: // methods
 
     void encode(eckit::Stream&) const override;
 
-    virtual bool optional() const override;
-    virtual const std::string &value(const Key &, const std::string &keyword) const override;
-    virtual const std::vector<std::string>& values(const metkit::mars::MarsRequest& rq, const std::string& keyword) const override;
-    virtual void print( std::ostream &out ) const override;
-    virtual const std::string &defaultValue() const override;
-    virtual void fill(Key &key, const std::string &keyword, const std::string& value) const override;
+    bool optional() const override;
+    const std::string& value(const Key&, const std::string& keyword) const override;
+    const std::vector<std::string>& values(const metkit::mars::MarsRequest& rq, const std::string& keyword) const override;
+    void print(std::ostream& out) const override;
+    const std::string& defaultValue() const override;
+    void fill(Key& key, const std::string& keyword, const std::string& value) const override;
 
 private: // members
 
