@@ -19,14 +19,12 @@
 #include <iosfwd>
 #include <vector>
 
-//#include "eckit/memory/NonCopyable.h"
 #include "eckit/serialisation/Streamable.h"
 
 class MarsTask;
-namespace metkit {
-namespace mars {
-    class MarsRequest;
-}
+
+namespace metkit::mars {
+class MarsRequest;
 }
 
 namespace fdb5 {
@@ -38,45 +36,43 @@ class TypesRegistry;
 
 class Matcher : public eckit::Streamable {
 
-public: // methods
+public:  // methods
+    Matcher() = default;
 
-    Matcher();
-    Matcher(eckit::Stream& s);
+    Matcher(eckit::Stream& stream);
 
-    virtual ~Matcher();
+    virtual bool optional() const { return false; }
 
-    virtual bool optional() const;
-
-    virtual const std::string &value(const Key &, const std::string &keyword) const;
+    virtual const std::string& value(const Key&, const std::string& keyword) const;
     virtual const std::vector<std::string>& values(const metkit::mars::MarsRequest& rq, const std::string& keyword) const;
-    virtual const std::string &defaultValue() const;
+    virtual const std::string& defaultValue() const;
 
-    virtual bool match(const std::string &keyword, const Key &key) const = 0;
-    virtual void fill(Key &key, const std::string &keyword, const std::string& value) const;
+    virtual bool match(const std::string& value) const                   = 0;
+    virtual bool match(const std::string& keyword, const Key& key) const = 0;
+    virtual void fill(Key& key, const std::string& keyword, const std::string& value) const;
 
+    virtual void dump(std::ostream& s, const std::string& keyword, const TypesRegistry& registry) const = 0;
 
-    virtual void dump(std::ostream &s, const std::string &keyword, const TypesRegistry &registry) const = 0;
+    friend std::ostream& operator<<(std::ostream& s, const Matcher& x);
 
-    friend std::ostream &operator<<(std::ostream &s, const Matcher &x);
+    // streamable
 
-//	const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
-	static const eckit::ClassSpec&  classSpec() { return classSpec_; }
+    static const eckit::ClassSpec& classSpec() { return classSpec_; }
 
-private: // methods
+private:  // methods
+    void encode(eckit::Stream& out) const override;
 
-    void encode(eckit::Stream&) const override;
+    virtual void print(std::ostream& out) const = 0;
 
-    virtual void print( std::ostream &out ) const = 0;
+private:  // members
+    // streamable
 
-private: // members
-
-    static eckit::ClassSpec classSpec_;
+    static eckit::ClassSpec           classSpec_;
     static eckit::Reanimator<Matcher> reanimator_;
-
 };
 
 //----------------------------------------------------------------------------------------------------------------------
 
-} // namespace fdb5
+}  // namespace fdb5
 
 #endif
