@@ -14,39 +14,42 @@
 #ifndef fdb5_SchemaParser_h
 #define fdb5_SchemaParser_h
 
+#include <iosfwd>
+#include <memory>
+#include <string>
+
 #include "eckit/parser/StreamParser.h"
 #include "eckit/types/Types.h"
 
-namespace fdb5 {
+#include "fdb5/rules/Rule.h"
 
-class Schema;
-class Rule;
-class Predicate;
-class TypesRegistry;
+namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
 class SchemaParser : public eckit::StreamParser {
 
-public: // methods
+public:  // methods
+    SchemaParser(std::istream& in) : StreamParser(in, true) { }
 
-    SchemaParser(std::istream &in);
-
-    void parse(const Schema &owner, std::vector<Rule *> &, TypesRegistry &registry);
-
-private: // methods
+    void parse(RuleList& result, TypesRegistry& registry);
 
+private:  // methods
     std::string parseIdent(bool value, bool emptyOK);
 
-    Rule *parseRule(const Schema &owner);
+    std::unique_ptr<RuleDatum> parseDatum();
 
-    Predicate *parsePredicate(std::map<std::string, std::string> &types);
-    void parseTypes(std::map<std::string, std::string> &);
+    std::unique_ptr<RuleIndex> parseIndex();
 
+    std::unique_ptr<RuleDatabase> parseDatabase();
+
+    std::unique_ptr<Predicate> parsePredicate(eckit::StringDict& types);
+
+    void parseTypes(eckit::StringDict& types);
 };
 
 //----------------------------------------------------------------------------------------------------------------------
 
-} // namespace eckit
+}  // namespace fdb5
 
 #endif
