@@ -11,8 +11,9 @@
 #include "eckit/exception/Exceptions.h"
 #include "eckit/log/Log.h"
 
-#include "fdb5/database/DB.h"
+#include "fdb5/database/Catalogue.h"
 #include "fdb5/toc/AdoptVisitor.h"
+#include "fdb5/toc/TocEngine.h"
 
 using namespace eckit;
 
@@ -20,8 +21,8 @@ namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-AdoptVisitor::AdoptVisitor(Archiver &owner, const Key &field, const PathName &path, Offset offset, Length length) :
-    BaseArchiveVisitor(owner, field),
+AdoptVisitor::AdoptVisitor(Archiver& owner, const Key& initialFieldKey, const PathName& path, Offset offset, Length length) :
+    BaseArchiveVisitor(owner, initialFieldKey),
     path_(path),
     offset_(offset),
     length_(length) {
@@ -29,19 +30,20 @@ AdoptVisitor::AdoptVisitor(Archiver &owner, const Key &field, const PathName &pa
     ASSERT(length_ > Length(0));
 }
 
-bool AdoptVisitor::selectDatum(const Key &key, const Key &full) {
+bool AdoptVisitor::selectDatum(const Key& datumKey, const Key& fullKey) {
+    checkMissingKeys(fullKey);
 
-    // Log::info() << "selectDatum " << key << ", " << full << " " << length_ << std::endl;
-    checkMissingKeys(full);
+    CatalogueWriter* cat = catalogue();
+    ASSERT(cat);
 
-    ASSERT(current());
-
-    current()->index(key, path_, offset_, length_);
-
-    return true;
+    if (cat->type() == TocEngine::typeName()) {
+        cat->index(datumKey, eckit::URI("file", path_), offset_, length_);
+        return true;
+    }
+    return false;
 }
 
-void AdoptVisitor::print(std::ostream &out) const {
+void AdoptVisitor::print(std::ostream& out) const {
     out << "AdoptVisitor["
         << "path=" << path_
         << ",offset=" << offset_
