@@ -32,17 +32,16 @@ class AdoptVisitor : public BaseArchiveVisitor {
 
 public: // methods
 
-    AdoptVisitor(Archiver &owner,
-                 const Key &field,
-                 const eckit::PathName &path,
+    AdoptVisitor(Archiver& owner,
+                 const Key& initialFieldKey,
+                 const eckit::PathName& path,
                  eckit::Offset offset,
                  eckit::Length length);
 
 protected: // methods
+    bool selectDatum(const Key& datumKey, const Key& fullKey) override;
 
-    virtual bool selectDatum(const InspectionKey &key, const Key &full) override;
-
-    virtual void print( std::ostream &out ) const override;
+    void print( std::ostream& out ) const override;
 
 private: // members
 
