@@ -59,22 +59,16 @@ public:
     /// but partial match for values
     bool match(const Key& k, const char* missing = 0) const {
 
-//        Log::debug<LibFDB>() << " Trying to key matching " << *this << " with key " << k << std::endl;
-
         if(k.size() != keyregex_.size()) return false;
 
         for(Key::const_iterator i = k.begin(); i != k.end(); ++i) {
 
-//            Log::debug<LibFDB>() << "     Match " << i->first << " " << i->second << std::endl;
-
             std::map<std::string, Regex>::const_iterator j = keyregex_.find(i->first);
 
             if(j == keyregex_.end()) {
                 return false;
             }
 
-//            Log::debug<LibFDB>() << "     Found " << j->first << " " << j->second << std::endl;
-
             if(!missing || i->second != missing) {
                 if(!j->second.match(i->second)) {
                     return false;
@@ -82,8 +76,6 @@ public:
             }
         }
 
-//        Log::debug<LibFDB>() << " Match successfull " << *this << " with key " << k << std::endl;
-
         return true;
     }
 
@@ -161,25 +153,22 @@ private: // methods
                     }
                     var = false;
 
-                    j = k.find(word);
-                    if(j != k.end()) {
-                        if(!missing) {
-                            result += (*j).second;
-                        }
-                        else {
-                            if((*j).second == missing || (*j).second.empty()) {
-                                result += keyregex_.find(word)->second; // we know it exists because it is ensured in match()
-                            }
-                            else
-                            {
+                    if (const auto [iter, found] = k.find(word); found) {
+                        if (!missing) {
+                            result += iter->second;
+                        } else {
+                            if (iter->second == missing || iter->second.empty()) {
+                                // we know it exists because it is ensured in match()
+                                result += keyregex_.find(word)->second;
+                            } else {
                                 result += (*j).second;
                             }
                         }
-                    }
-                    else {
-                        std::ostringstream os;
-                        os << "FDB RootManager substituteVars: cannot find a value for '" << word << "' in " <<s << " at position " << i;
-                        throw UserError(os.str());
+                    } else {
+                        std::ostringstream oss;
+                        oss << "FDB RootManager substituteVars: cannot find a value for '" << word << "' in " << s
+                            << " at position " << i;
+                        throw UserError(oss.str());
                     }
                     break;
 
@@ -558,11 +547,7 @@ FileSpaceTable RootManager::fileSpaces() {
 
 RootManager::RootManager(const Config& config) :
     dbPathNamers_(readDbNamers(config)),
-    config_(config) {
-
-//    eckit::Log::info() << "Root manager: " << spacesTable_ << std::endl;
-}
-
+    config_(config) {}
 
 std::string RootManager::dbPathName(const Key& key)
 {
@@ -596,11 +581,10 @@ std::vector<std::string> RootManager::possibleDbPathNames(const Key& key, const
 
     std::ostringstream oss;
     const char *sep = "";
-
-    for (auto& k : key.names()) {
-        auto& v = key.get(k);
+    for (const auto& k : key.names()) {
+        const auto& v = key.get(k);
         oss << sep;
-        oss << (v == missing || v.empty() ? missing : key.canonicalValue(k));
+        oss << (v == missing || v.empty() ? missing : v);
         sep = ":";
     }
     result.push_back(oss.str());
@@ -631,9 +615,9 @@ TocPath RootManager::directory(const Key& key) {
 
     for (FileSpaceTable::const_iterator i = spacesTable_.begin(); i != spacesTable_.end() ; ++i) {
         if(i->match(keystr)) {
-            TocPath root = i->filesystem(key, dbpath);
-            LOG_DEBUG_LIB(LibFdb5) << "Directory root " << root.directory_ << " dbpath " << dbpath <<  std::endl;
-            return TocPath{root.directory_ / dbpath, root.controlIdentifiers_};
+            TocPath db = i->filesystem(key, dbpath);
+            LOG_DEBUG_LIB(LibFdb5) << "Database directory " << db.directory_ << std::endl;
+            return db;
         }
     }
 
@@ -642,21 +626,6 @@ TocPath RootManager::directory(const Key& key) {
     throw eckit::SeriousBug(oss.str());
 }
 
-std::vector<PathName> RootManager::allRoots(const Key& key)
-{
-    eckit::StringSet roots;
-
-    std::string k = key.valuesToString();
-
-    for (FileSpaceTable::const_iterator i = spacesTable_.begin(); i != spacesTable_.end() ; ++i) {
-        if(i->match(k)) {
-            i->all(roots);
-        }
-    }
-
-    return std::vector<eckit::PathName>(roots.begin(), roots.end());
-}
-
 std::vector<PathName> RootManager::visitableRoots(const std::set<Key>& keys) {
 
     eckit::StringSet roots;
@@ -694,10 +663,11 @@ std::vector<eckit::PathName> RootManager::visitableRoots(const Key& key) {
 
 std::vector<eckit::PathName> RootManager::visitableRoots(const metkit::mars::MarsRequest& request) {
 
-//    Key key;
-//    config_.schema().expandFirstLevel(request, key);
+    std::map<Key, const Rule*> results;
     std::set<Key> keys;
-    config_.schema().matchFirstLevel(request, keys, "");
+    config_.schema().matchDatabase(request, results, "");
+    for (const auto& [key, rule] : results)
+        keys.insert(key);
     return visitableRoots(keys);
 }
 
