@@ -26,19 +26,29 @@ namespace fdb5 {
 //----------------------------------------------------------------------------------------------------------------------
 
 TocCatalogue::TocCatalogue(const Key& key, const fdb5::Config& config) :
-    TocCatalogue(key, CatalogueRootManager(config).directory(key), config) {}
+    TocCatalogue(key, CatalogueRootManager(config).directory(key), config) {
+}
 
 TocCatalogue::TocCatalogue(const Key& key, const TocPath& tocPath, const fdb5::Config& config) :
-    Catalogue(key, tocPath.controlIdentifiers_, config),
-    TocHandler(tocPath.directory_, config) {}
+    CatalogueImpl(key, tocPath.controlIdentifiers_, config),
+    TocHandler(tocPath.directory_, config) {
+}
 
 TocCatalogue::TocCatalogue(const eckit::PathName& directory, const ControlIdentifiers& controlIdentifiers, const fdb5::Config& config) :
-    Catalogue(Key(), controlIdentifiers, config),
+    CatalogueImpl(Key(), controlIdentifiers, config),
     TocHandler(directory, config) {
     // Read the real DB key into the DB base object
     dbKey_ = databaseKey();
 }
 
+
+std::vector<Index> TocCatalogue::loadIndexes(bool sorted,
+                                           std::set<std::string>* subTocs,
+                                           std::vector<bool>* indexInSubtoc,
+                                           std::vector<Key>* remapKeys) const {
+    return TocHandler::loadIndexes(*this, sorted, subTocs, indexInSubtoc,remapKeys);
+}
+
 bool TocCatalogue::exists() const {
     return TocHandler::exists();
 }
@@ -61,8 +71,9 @@ const Schema& TocCatalogue::schema() const {
     return *schema_;
 }
 
-const eckit::PathName& TocCatalogue::basePath() const {
-    return directory_;
+const Rule& TocCatalogue::rule() const {
+    ASSERT(rule_);
+    return *rule_;
 }
 
 std::vector<PathName> TocCatalogue::metadataPaths() const {
@@ -81,6 +92,7 @@ std::vector<PathName> TocCatalogue::metadataPaths() const {
 void TocCatalogue::loadSchema() {
     Timer timer("TocCatalogue::loadSchema()", Log::debug<LibFdb5>());
     schema_ = &SchemaRegistry::instance().get(schemaPath());
+    rule_ = &schema_->matchingRule(dbKey_);
 }
 
 StatsReportVisitor* TocCatalogue::statsReportVisitor() const {
@@ -110,12 +122,12 @@ std::vector<Index> TocCatalogue::indexes(bool sorted) const {
 
 void TocCatalogue::allMasked(std::set<std::pair<URI, Offset>>& metadata,
                       std::set<URI>& data) const {
-    enumerateMasked(metadata, data);
+    enumerateMasked(*this, metadata, data);
 }
 
 std::string TocCatalogue::type() const
 {
-    return TocCatalogue::catalogueTypeName();
+    return TocEngine::typeName();
 }
 
 void TocCatalogue::checkUID() const {
@@ -139,7 +151,7 @@ void TocCatalogue::control(const ControlAction& action, const ControlIdentifiers
 }
 
 bool TocCatalogue::enabled(const ControlIdentifier& controlIdentifier) const {
-    return Catalogue::enabled(controlIdentifier) && TocHandler::enabled(controlIdentifier);
+    return CatalogueImpl::enabled(controlIdentifier) && TocHandler::enabled(controlIdentifier);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
