@@ -39,9 +39,9 @@ public:
 
     eckit::DataHandle* dataHandle() const override;
 
-    virtual std::shared_ptr<FieldLocation> make_shared() const override;
+    std::shared_ptr<const FieldLocation> make_shared() const override;
 
-    virtual void visit(FieldLocationVisitor& visitor) const override;
+    void visit(FieldLocationVisitor& visitor) const override;
 
 public: // For Streamable
 
@@ -49,8 +49,8 @@ public: // For Streamable
 
 protected: // For Streamable
 
-    virtual const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
-    virtual void encode(eckit::Stream&) const override;
+    const eckit::ReanimatorBase& reanimator() const override { return reanimator_; }
+    void encode(eckit::Stream&) const override;
 
     static eckit::ClassSpec                    classSpec_;
     static eckit::Reanimator<TocFieldLocation> reanimator_;
