@@ -16,8 +16,10 @@
 
 #include <map>
 #include <memory>
+#include <utility>
 
 #include "eckit/filesystem/PathName.h"
+#include "eckit/filesystem/LocalPathName.h"
 #include "eckit/filesystem/URI.h"
 #include "eckit/io/Length.h"
 #include "eckit/io/MemoryHandle.h"
@@ -25,7 +27,7 @@
 
 #include "fdb5/config/Config.h"
 #include "fdb5/database/DbStats.h"
-#include "fdb5/database/DB.h"
+#include "fdb5/database/Catalogue.h"
 #include "fdb5/toc/TocCommon.h"
 #include "fdb5/toc/TocRecord.h"
 #include "fdb5/toc/TocSerialisationVersion.h"
@@ -34,6 +36,7 @@
 
 namespace eckit {
 class Configuration;
+class MemoryHandle;
 }
 
 namespace fdb5 {
@@ -98,23 +101,20 @@ class TocHandler : public TocCommon, private eckit::NonCopyable {
 public: // typedefs
 
     typedef std::vector<TocRecord> TocVec;
-    typedef std::vector< eckit::PathName > TocPaths;
+    typedef std::vector< eckit::LocalPathName > TocPaths;
 
 public: // methods
-
-    TocHandler( const Key &key, const Config& config);
-
     TocHandler( const eckit::PathName &dir, const Config& config);
 
     /// For initialising sub tocs or diagnostic interrogation.
-    TocHandler(const eckit::PathName& path, const Key& parentKey);
+    TocHandler(const eckit::PathName& path, const Key& parentKey, eckit::MemoryHandle* cachedToc=nullptr);
 
     ~TocHandler() override;
 
     bool exists() const;
     void checkUID() const override;
 
-    void writeInitRecord(const Key &tocKey);
+    void writeInitRecord(const Key& tocKey);
     void writeClearRecord(const Index &);
     void writeClearAllRecord();
     void writeSubTocRecord(const TocHandler& subToc);
@@ -128,7 +128,8 @@ public: // methods
 
     /// Return a list of existent indexes. If supplied, also supply a list of associated
     /// subTocs that were read to get these indexes
-    std::vector<Index> loadIndexes(bool sorted=false,
+    std::vector<Index> loadIndexes(const Catalogue& catalogue,
+                                   bool sorted=false,
                                    std::set<std::string>* subTocs = nullptr,
                                    std::vector<bool>* indexInSubtoc = nullptr,
                                    std::vector<Key>* remapKeys = nullptr) const;
@@ -136,9 +137,9 @@ public: // methods
     Key databaseKey();
     size_t numberOfRecords() const;
 
-    const eckit::PathName& directory() const;
-    const eckit::PathName& tocPath() const;
-    const eckit::PathName& schemaPath() const;
+    const eckit::LocalPathName& directory() const;
+    const eckit::LocalPathName& tocPath() const;
+    const eckit::LocalPathName& schemaPath() const;
 
     void dump(std::ostream& out, bool simple = false, bool walkSubTocs = true) const;
     void dumpIndexFile(std::ostream& out, const eckit::PathName& indexFile) const;
@@ -146,7 +147,8 @@ public: // methods
 
     DbStats stats() const;
 
-    void enumerateMasked(std::set<std::pair<eckit::URI, eckit::Offset>>& metadata,
+    void enumerateMasked(const Catalogue& catalogue,
+                         std::set<std::pair<eckit::URI, eckit::Offset>>& metadata,
                          std::set<eckit::URI>& data) const;
 
     std::vector<eckit::PathName> subTocPaths() const;
@@ -166,7 +168,7 @@ protected: // methods
 
 private: // methods
 
-    eckit::PathName fullControlFilePath(const std::string& name) const;
+    eckit::LocalPathName fullControlFilePath(const std::string& name) const;
     void createControlFile(const std::string& name) const;
     void removeControlFile(const std::string& name) const;
 
@@ -179,8 +181,8 @@ protected: // members
 protected: // methods
 
     // Handle location and remapping information if using a mounted TocCatalogue
-    const eckit::PathName& currentDirectory() const;
-    const eckit::PathName& currentTocPath() const;
+    const eckit::LocalPathName& currentDirectory() const;
+    const eckit::LocalPathName& currentTocPath() const;
     const Key& currentRemapKey() const;
 
     // Build the record, and return the payload size
@@ -192,9 +194,10 @@ protected: // methods
 
     // Given the payload size, returns the record size
 
-    static size_t roundRecord(TocRecord &r, size_t payloadSize);
+    static std::pair<size_t, size_t> recordSizes(TocRecord &r, size_t payloadSize);
 
     void appendBlock(const void* data, size_t size);
+    void appendBlock(TocRecord &r, size_t payloadSize);
 
     const TocSerialisationVersion& serialisationVersion() const;
 
@@ -208,12 +211,17 @@ private: // methods
 
     void close() const;
 
+    void appendRaw(const void* data, size_t size);
+    void appendRound(TocRecord &r, size_t payloadSize);
+
     /// Populate the masked sub toc list, starting from the _current_position_ in the
     /// file (opened for read). It resets back to the same place when done. This is
     /// to allow searching only from the first subtoc.
     void allMaskableEntries(eckit::Offset startOffset, eckit::Offset endOffset,
-                            std::set<std::pair<eckit::PathName, eckit::Offset>>& maskedEntries) const;
+                            std::set<std::pair<eckit::LocalPathName, eckit::Offset>>& maskedEntries) const;
+    eckit::LocalPathName parseSubTocRecord(const TocRecord& r, bool readMasked) const;
     void populateMaskedEntriesList() const;
+    void preloadSubTocs(bool readMasked) const;
 
     void append(TocRecord &r, size_t payloadSize);
 
@@ -222,9 +230,12 @@ private: // methods
     // readMasked=true will walk subtocs and read indexes even if they are masked. This is
     // useful for dumping indexes which are cleared, or only referred to in cleared subtocs.
     bool readNext(TocRecord &r, bool walkSubTocs = true, bool hideSubTocEntries = true,
-                  bool hideClearEntries = true, bool readMasked = false) const;
+                  bool hideClearEntries = true, bool readMasked = false,
+                  const TocRecord** data=nullptr, size_t* length=nullptr) const;
 
-    bool readNextInternal(TocRecord &r) const;
+    void selectSubTocRead(const eckit::LocalPathName& path) const;
+
+    bool readNextInternal(TocRecord &r, const TocRecord** data=nullptr, size_t* length=nullptr) const;
 
     std::string userName(long) const;
 
@@ -234,7 +245,7 @@ private: // methods
 
 private: // members
 
-    eckit::PathName tocPath_;
+    eckit::LocalPathName tocPath_;
     Config dbConfig_;
 
     TocSerialisationVersion serialisationVersion_;
@@ -253,14 +264,18 @@ private: // members
     mutable std::unique_ptr<eckit::MemoryHandle> cachedToc_; ///< this is only for read path
 
     /// The sub toc is initialised in the read or write pathways for maintaining state.
-    mutable std::unique_ptr<TocHandler> subTocRead_;
+    mutable std::map<eckit::LocalPathName, std::unique_ptr<TocHandler>> subTocReadCache_;
+    mutable TocHandler* subTocRead_; // n.b. non-owning
     mutable std::unique_ptr<TocHandler> subTocWrite_;
     mutable size_t count_;
 
-    mutable std::set<std::pair<eckit::PathName, eckit::Offset>> maskedEntries_;
+    mutable std::set<std::pair<eckit::LocalPathName, eckit::Offset>> maskedEntries_;
 
     mutable bool enumeratedMaskedEntries_;
+    mutable int numSubtocsRaw_;
     mutable bool writeMode_;
+
+    mutable bool dirty_;
 };
 
 
