@@ -8,14 +8,12 @@
  * does it submit to any jurisdiction.
  */
 
-#include "eckit/log/BigNum.h"
-
-#include "fdb5/LibFdb5.h"
-#include "fdb5/toc/TocStats.h"
 #include "fdb5/toc/TocIndex.h"
+#include "fdb5/LibFdb5.h"
 #include "fdb5/toc/BTreeIndex.h"
 #include "fdb5/toc/FieldRef.h"
 #include "fdb5/toc/TocFieldLocation.h"
+#include "fdb5/toc/TocStats.h"
 
 namespace fdb5 {
 
@@ -41,50 +39,58 @@ public:
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/// @note We use a FileStoreWrapper base that only exists to initialise the files_ member function
+/// @note We use a FileStoreWrapper base that only exists to initialise the uris_ member function
 ///       before the Index constructor is called. This is necessary as due to (preexisting)
-///       serialisation ordering, the files_ member needs to be initialised from a Stream
+///       serialisation ordering, the uris_ member needs to be initialised from a Stream
 ///       before the type_ members of Index, but Indexs WILL be constructed before
 ///       the members of TocIndex
 
-TocIndex::TocIndex(const Key &key, const eckit::PathName &path, off_t offset, Mode mode, const std::string& type ) :
-    UriStoreWrapper(path.dirName()),
-    IndexBase(key, type),
-    btree_(nullptr),
-    dirty_(false),
-    mode_(mode),
-    location_(path, offset),
-    preloadBTree_(false) {
-}
+TocIndex::TocIndex(const Key&             key,
+                   const Catalogue&       catalogue,
+                   const eckit::PathName& path,
+                   off_t                  offset,
+                   Mode                   mode,
+                   const std::string&     type)
+    : UriStoreWrapper(path.dirName()),
+      IndexBase(key, type),
+      btree_(nullptr),
+      dirty_(false),
+      mode_(mode),
+      location_(path, offset),
+      preloadBTree_(false) { }
 
-TocIndex::TocIndex(eckit::Stream &s, const int version, const eckit::PathName &directory, const eckit::PathName &path,
-                   off_t offset, bool preloadBTree):
-    UriStoreWrapper(directory, s),
-    IndexBase(s, version),
-    btree_(nullptr),
-    dirty_(false),
-    mode_(TocIndex::READ),
-    location_(path, offset),
-    preloadBTree_(preloadBTree) {
-}
+TocIndex::TocIndex(eckit::Stream&         s,
+                   const Catalogue&       catalogue,
+                   const int              version,
+                   const eckit::PathName& directory,
+                   const eckit::PathName& path,
+                   off_t                  offset,
+                   bool                   preloadBTree)
+    : UriStoreWrapper(directory, s),
+      IndexBase(s, version),
+      btree_(nullptr),
+      dirty_(false),
+      mode_(TocIndex::READ),
+      location_(path, offset),
+      preloadBTree_(preloadBTree) { }
 
 TocIndex::~TocIndex() {
     close();
 }
 
 void TocIndex::encode(eckit::Stream& s, const int version) const {
-    files_.encode(s);
+    uris_.encode(s);
     IndexBase::encode(s, version);
 }
 
 
-bool TocIndex::get(const Key &key, const Key &remapKey, Field &field) const {
+bool TocIndex::get(const Key& key, const Key& remapKey, Field &field) const {
     ASSERT(btree_);
     FieldRef ref;
 
     bool found = btree_->get(key.valuesToString(), ref);
     if ( found ) {
-        const eckit::URI& uri = files_.get(ref.uriId());
+        const eckit::URI& uri = uris_.get(ref.uriId());
         FieldLocation* loc = FieldLocationFactory::instance().build(uri.scheme(), uri, ref.offset(), ref.length(), remapKey);
         field = Field(std::move(*loc), timestamp_, ref.details());
         delete(loc);
@@ -124,11 +130,11 @@ void TocIndex::close() {
     }
 }
 
-void TocIndex::add(const Key &key, const Field &field) {
+void TocIndex::add(const Key& key, const Field &field) {
     ASSERT(btree_);
     ASSERT( mode_ == TocIndex::WRITE );
 
-    FieldRef ref(files_, field);
+    FieldRef ref(uris_, field);
 
     //  bool replace =
     btree_->set(key.valuesToString(), ref); // returns true if replace, false if new insert
@@ -165,15 +171,16 @@ void TocIndex::funlock() const {
 }
 
 class TocIndexVisitor : public BTreeIndexVisitor {
-    const UriStore &files_;
+    const UriStore &uris_;
     EntryVisitor &visitor_;
 public:
-    TocIndexVisitor(const UriStore &files, EntryVisitor &visitor):
-        files_(files),
+    TocIndexVisitor(const UriStore &uris, EntryVisitor &visitor):
+        uris_(uris),
         visitor_(visitor) {}
 
     void visit(const std::string& keyFingerprint, const FieldRef& ref) {
-        Field field(TocFieldLocation(files_, ref), visitor_.indexTimestamp(), ref.details());
+
+        Field field(TocFieldLocation(uris_, ref), visitor_.indexTimestamp(), ref.details());
         visitor_.visitDatum(field, keyFingerprint);
     }
 };
@@ -185,7 +192,7 @@ void TocIndex::entries(EntryVisitor &visitor) const {
     // Allow the visitor to selectively decline to visit the entries in this index
     if (visitor.visitIndex(instantIndex)) {
         TocIndexCloser closer(*this);
-        TocIndexVisitor v(files_, visitor);
+        TocIndexVisitor v(uris_, visitor);
         btree_->visit(v);
     }
 }
@@ -199,8 +206,8 @@ std::string TocIndex::defaulType() {
     return BTreeIndex::defaulType();
 }
 
-const std::vector<eckit::URI> TocIndex::dataURIs() const {
-    return files_.paths();
+std::vector<eckit::URI> TocIndex::dataURIs() const {
+    return uris_.paths();
 }
 
 bool TocIndex::dirty() const {
@@ -228,7 +235,7 @@ void TocIndex::dump(std::ostream &out, const char* indent, bool simple, bool dum
 
     if(!simple) {
         out << std::endl;
-        files_.dump(out, indent);
+        uris_.dump(out, indent);
         axes_.dump(out, indent);
     }
 
