@@ -35,17 +35,17 @@ namespace fdb5 {
 class BTreeIndex;
 
 
-/// FileStoreWrapper exists _only_ so that the files_ member can be initialised from the stream
+/// FileStoreWrapper exists _only_ so that the uris_ member can be initialised from the stream
 /// before the Index base class is initialised, for the TocIndex class. This order is required
-/// to preserve the order that data is stored/read from streams from before the files_ object
+/// to preserve the order that data is stored/read from streams from before the uris_ object
 /// was moved into the TocIndex class.
 
 struct UriStoreWrapper {
 
-    UriStoreWrapper(const eckit::PathName& directory) : files_(directory) {}
-    UriStoreWrapper(const eckit::PathName& directory, eckit::Stream& s) : files_(directory, s) {}
+    UriStoreWrapper(const eckit::PathName& directory) : uris_(directory) {}
+    UriStoreWrapper(const eckit::PathName& directory, eckit::Stream& s) : uris_(directory, s) {}
 
-    UriStore files_;
+    UriStore uris_;
 };
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -61,13 +61,15 @@ public: // types
 
 public: // methods
 
-    TocIndex(const Key &key,
+    TocIndex(const Key& key,
+             const Catalogue& catalogue,
              const eckit::PathName &path,
              off_t offset,
              Mode mode,
              const std::string& type = defaulType());
 
     TocIndex(eckit::Stream &,
+             const Catalogue& catalogue,
              const int version,
              const eckit::PathName &directory,
              const eckit::PathName &path,
@@ -83,11 +85,11 @@ public: // methods
 
     void flock() const override;
     void funlock() const override;
-    
-private: // methods
 
+private: // methods
     const IndexLocation& location() const override { return location_; }
-    const std::vector<eckit::URI> dataURIs() const override;
+
+    std::vector<eckit::URI> dataURIs() const override;
 
     bool dirty() const override;
 
@@ -97,8 +99,8 @@ private: // methods
 
     void visit(IndexLocationVisitor& visitor) const override;
 
-    bool get( const Key &key, const Key &remapKey, Field &field ) const override;
-    void add( const Key &key, const Field &field ) override;
+    bool get(const Key& key, const Key& remapKey, Field& field ) const override;
+    void add(const Key& key, const Field& field ) override;
     void flush() override;
     void encode(eckit::Stream& s, const int version) const override;
     void entries(EntryVisitor& visitor) const override;
