@@ -34,8 +34,7 @@ public:
     ~TocMoveVisitor() override;
 
 private: // methods
-
-    bool visitDatabase(const Catalogue& catalogue, const Store& store) override;
+    bool visitDatabase(const Catalogue& catalogue) override;
 
     void move();
 
