@@ -189,8 +189,8 @@ TocDataStats& TocDataStats::operator+=(const TocDataStats& rhs) {
 
     std::swap(activeDataFiles_, intersect);
 
-    for(std::map<eckit::PathName, size_t>::const_iterator i = rhs.dataUsage_.begin(); i != rhs.dataUsage_.end(); ++i) {
-        dataUsage_[i->first] += i->second;
+    for(const auto& [path, size] : rhs.dataUsage_) {
+        dataUsage_[path] += size;
     }
 
     return *this;
@@ -218,7 +218,6 @@ TocStatsReportVisitor::TocStatsReportVisitor(const TocCatalogue& catalogue, bool
 
 TocStatsReportVisitor::~TocStatsReportVisitor() {}
 
-//bool TocStatsReportVisitor::visitDatabase(const Catalogue& catalogue, const Store& store) {
 bool TocStatsReportVisitor::visitDatabase(const Catalogue& catalogue) {
     ASSERT(&catalogue == currentCatalogue_);
     return true;
@@ -227,8 +226,6 @@ bool TocStatsReportVisitor::visitDatabase(const Catalogue& catalogue) {
 
 void TocStatsReportVisitor::visitDatum(const Field& field, const std::string& fieldFingerprint) {
 
-//    ASSERT(currIndex_ != 0);
-
     TocDbStats* dbStats = new TocDbStats();
 
     // Exclude non-owned data if relevant
@@ -258,19 +255,19 @@ void TocStatsReportVisitor::visitDatum(const Field& field, const std::string& fi
     const eckit::PathName& indexPath = currentIndex_->location().uri().path();
 
     if (dataPath != lastDataPath_) {
+        if (dataPath.exists()) {
+            if (allDataFiles_.find(dataPath) == allDataFiles_.end()) {
 
-        if (allDataFiles_.find(dataPath) == allDataFiles_.end()) {
-
-            if (dataPath.dirName().sameAs(directory_)) {
-                dbStats->ownedFilesSize_ += dataPath.size();
-                dbStats->ownedFilesCount_++;
-            } else {
-                dbStats->adoptedFilesSize_ += dataPath.size();
-                dbStats->adoptedFilesCount_++;
+                if (dataPath.dirName().sameAs(directory_)) {
+                    dbStats->ownedFilesSize_ += dataPath.size();
+                    dbStats->ownedFilesCount_++;
+                } else {
+                    dbStats->adoptedFilesSize_ += dataPath.size();
+                    dbStats->adoptedFilesCount_++;
+                }
+                allDataFiles_.insert(dataPath);
             }
-            allDataFiles_.insert(dataPath);
         }
-
         lastDataPath_ = dataPath;
     }
 
