@@ -10,6 +10,7 @@
 
 #include <dirent.h>
 #include <fcntl.h>
+#include <memory>
 
 #include "eckit/log/Timer.h"
 
@@ -34,10 +35,10 @@ namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-TocStore::TocStore(const Schema& schema, const Key& key, const Config& config) :
-    Store(schema),
+TocStore::TocStore(const Key& key, const Config& config) :
+    Store(),
     TocCommon(StoreRootManager(config).directory(key).directory_),
-    auxFileExtensions_{auxFileExtensions()} {}
+    archivedFields_(0), auxFileExtensions_{auxFileExtensions()} {}
 
 eckit::URI TocStore::uri() const {
 
@@ -77,7 +78,7 @@ std::vector<eckit::URI> TocStore::collocatedDataURIs() const {
 
     std::vector<eckit::PathName> files;
     std::vector<eckit::PathName> dirs;
-    (directory_).children(files, dirs);
+    PathName(directory_).children(files, dirs);
 
     std::vector<eckit::URI> res;
     for (const auto& f : files) {
@@ -115,10 +116,10 @@ eckit::DataHandle* TocStore::retrieve(Field& field) const {
     return field.dataHandle();
 }
 
-std::unique_ptr<FieldLocation> TocStore::archive(const Key &key, const void *data, eckit::Length length) {
-    dirty_ = true;
+std::unique_ptr<const FieldLocation> TocStore::archive(const Key& idxKey, const void* data, eckit::Length length) {
+    archivedFields_++;
 
-    eckit::PathName dataPath = getDataPath(key);
+    eckit::PathName dataPath = getDataPath(idxKey);
 
     eckit::DataHandle &dh = getDataHandle(dataPath);
 
@@ -128,19 +129,21 @@ std::unique_ptr<FieldLocation> TocStore::archive(const Key &key, const void *dat
 
     ASSERT(len == length);
 
-    return std::unique_ptr<TocFieldLocation>(new TocFieldLocation(dataPath, position, length, Key(nullptr, true)));
+    return std::make_unique<TocFieldLocation>(dataPath, position, length, Key());
 }
 
-void TocStore::flush() {
-    if (!dirty_) {
-        return;
+size_t TocStore::flush() {
+    if (archivedFields_ == 0) {
+        return 0;
     }
 
     // ensure consistent state before writing Toc entry
-
     flushDataHandles();
 
-    dirty_ = false;
+    size_t out = archivedFields_;
+    archivedFields_ = 0;
+
+    return out;
 }
 
 void TocStore::close() {
@@ -162,24 +165,23 @@ void TocStore::remove(const eckit::URI& uri, std::ostream& logAlways, std::ostre
     }
 }
 
-eckit::DataHandle *TocStore::getCachedHandle( const eckit::PathName &path ) const {
-    HandleStore::const_iterator j = handles_.find( path );
-    if ( j != handles_.end() )
-        return j->second;
-    else
-        return nullptr;
+eckit::DataHandle* TocStore::getCachedHandle( const eckit::PathName &path ) const {
+    std::lock_guard lock(handlesMutex_);
+    if (auto j = handles_.find(path); j != handles_.end()) {
+        return j->second.get();
+    }
+    return nullptr;
 }
 
 void TocStore::closeDataHandles() {
-    for ( HandleStore::iterator j = handles_.begin(); j != handles_.end(); ++j ) {
-        eckit::DataHandle *dh = j->second;
+    std::lock_guard lock(handlesMutex_);
+    for (const auto& [p, dh] : handles_) {
         dh->close();
-        delete dh;
     }
     handles_.clear();
 }
 
-eckit::DataHandle *TocStore::createFileHandle(const eckit::PathName &path) {
+std::unique_ptr<eckit::DataHandle> TocStore::createFileHandle(const eckit::PathName &path) {
 
     static size_t sizeBuffer = eckit::Resource<unsigned long>("fdbBufferSize", 64 * 1024 * 1024);
 
@@ -189,17 +191,17 @@ eckit::DataHandle *TocStore::createFileHandle(const eckit::PathName &path) {
                                      << " buffer size " << sizeBuffer
                                      << std::endl;
 
-        return new LustreFileHandle<FDBFileHandle>(path, sizeBuffer, stripeDataLustreSettings());
+        return std::make_unique<LustreFileHandle<FDBFileHandle>>(path, sizeBuffer, stripeDataLustreSettings());
     }
 
     LOG_DEBUG_LIB(LibFdb5) << "Creating FDBFileHandle to " << path
                                  << " with buffer of " << eckit::Bytes(sizeBuffer)
                                  << std::endl;
 
-    return new FDBFileHandle(path, sizeBuffer);
+    return std::make_unique<FDBFileHandle>(path, sizeBuffer);
 }
 
-eckit::DataHandle *TocStore::createAsyncHandle(const eckit::PathName &path) {
+std::unique_ptr<eckit::DataHandle> TocStore::createAsyncHandle(const eckit::PathName &path) {
 
     static size_t nbBuffers  = eckit::Resource<unsigned long>("fdbNbAsyncBuffers", 4);
     static size_t sizeBuffer = eckit::Resource<unsigned long>("fdbSizeAsyncBuffer", 64 * 1024 * 1024);
@@ -211,17 +213,18 @@ eckit::DataHandle *TocStore::createAsyncHandle(const eckit::PathName &path) {
                                      << " buffer each with " << eckit::Bytes(sizeBuffer)
                                      << std::endl;
 
-        return new LustreFileHandle<eckit::AIOHandle>(path, nbBuffers, sizeBuffer, stripeDataLustreSettings());
+        return std::make_unique<LustreFileHandle<eckit::AIOHandle>>(path, nbBuffers, sizeBuffer, stripeDataLustreSettings());
     }
 
-    return new eckit::AIOHandle(path, nbBuffers, sizeBuffer);
+    return std::make_unique<eckit::AIOHandle>(path, nbBuffers, sizeBuffer);
 }
 
-eckit::DataHandle *TocStore::createDataHandle(const eckit::PathName &path) {
+std::unique_ptr<eckit::DataHandle> TocStore::createDataHandle(const eckit::PathName &path) {
 
     static bool fdbWriteToNull = eckit::Resource<bool>("fdbWriteToNull;$FDB_WRITE_TO_NULL", false);
-    if(fdbWriteToNull)
-        return new eckit::EmptyHandle();
+    if(fdbWriteToNull) {
+        return std::make_unique<eckit::EmptyHandle>();
+    }
 
     static bool fdbAsyncWrite = eckit::Resource<bool>("fdbAsyncWrite;$FDB_ASYNC_WRITE", false);
     if(fdbAsyncWrite)
@@ -231,17 +234,18 @@ eckit::DataHandle *TocStore::createDataHandle(const eckit::PathName &path) {
 }
 
 eckit::DataHandle& TocStore::getDataHandle( const eckit::PathName &path ) {
-    eckit::DataHandle *dh = getCachedHandle(path);
-    if ( !dh ) {
-        dh = createDataHandle(path);
-        ASSERT(dh);
-        handles_[path] = dh;
-        dh->openForAppend(0);
+    std::lock_guard lock(handlesMutex_);
+    eckit::DataHandle* dh = getCachedHandle(path);
+    if (dh) {
+        return *dh;
     }
-    return *dh;
+    auto dataHandle = createDataHandle(path);
+    ASSERT(dataHandle);
+    dataHandle->openForAppend(0);
+    return *(handles_[path] = std::move(dataHandle));
 }
 
-eckit::PathName TocStore::generateDataPath(const Key &key) const {
+eckit::PathName TocStore::generateDataPath(const Key& key) const {
 
     eckit::PathName dpath ( directory_ );
     dpath /=  key.valuesToString();
@@ -249,7 +253,7 @@ eckit::PathName TocStore::generateDataPath(const Key &key) const {
     return dpath;
 }
 
-eckit::PathName TocStore::getDataPath(const Key &key) const {
+eckit::PathName TocStore::getDataPath(const Key& key) const {
     PathStore::const_iterator j = dataPaths_.find(key);
     if ( j != dataPaths_.end() )
         return j->second;
@@ -262,9 +266,8 @@ eckit::PathName TocStore::getDataPath(const Key &key) const {
 }
 
 void TocStore::flushDataHandles() {
-
-    for (HandleStore::iterator j = handles_.begin(); j != handles_.end(); ++j) {
-        eckit::DataHandle *dh = j->second;
+    std::lock_guard lock(handlesMutex_);
+    for (const auto& [p, dh] : handles_) {
         dh->flush();
     }
 }
@@ -321,7 +324,7 @@ void TocStore::moveTo(const Key& key, const Config& config, const eckit::URI& de
     eckit::PathName destPath = dest.path();
 
     for (const eckit::PathName& root: StoreRootManager(config).canMoveToRoots(key)) {
-        if (root.sameAs(destPath)) {      
+        if (root.sameAs(destPath)) {
             eckit::PathName src_db = directory_;
             eckit::PathName dest_db = destPath / key.valuesToString();
 
@@ -351,7 +354,7 @@ void TocStore::moveTo(const Key& key, const Config& config, const eckit::URI& de
 void TocStore::remove(const Key& key) const {
 
     eckit::PathName src_db = directory_;
-        
+
     DIR* dirp = ::opendir(src_db.asString().c_str());
     struct dirent* dp;
     while ((dp = ::readdir(dirp)) != NULL) {
