@@ -16,12 +16,16 @@
 #ifndef fdb5_TocStore_H
 #define fdb5_TocStore_H
 
-#include "fdb5/database/DB.h"
+#include <map>
+#include <memory>
+#include <mutex>
+#include <set>
+
+#include "fdb5/database/Catalogue.h"
 #include "fdb5/database/Index.h"
 #include "fdb5/database/Store.h"
 #include "fdb5/rules/Schema.h"
 #include "fdb5/toc/TocCommon.h"
-#include "fdb5/toc/TocEngine.h"
 
 namespace fdb5 {
 
@@ -33,7 +37,7 @@ class TocStore : public Store, public TocCommon {
 
 public: // methods
 
-    TocStore(const Schema& schema, const Key& key, const Config& config);
+    TocStore(const Key& key, const Config& config);
 
     ~TocStore() override {}
 
@@ -44,7 +48,7 @@ public: // methods
     std::set<eckit::URI> asCollocatedDataURIs(const std::vector<eckit::URI>&) const override;
 
     bool open() override { return true; }
-    void flush() override;
+    size_t flush() override;
     void close() override;
 
     void checkUID() const override { TocCommon::checkUID(); }
@@ -64,36 +68,37 @@ protected: // methods
     bool exists() const override;
 
     eckit::DataHandle* retrieve(Field& field) const override;
-    std::unique_ptr<FieldLocation> archive(const Key &key, const void *data, eckit::Length length) override;
+    std::unique_ptr<const FieldLocation> archive(const Key& idxKey, const void *data, eckit::Length length) override;
 
     void remove(const eckit::URI& uri, std::ostream& logAlways, std::ostream& logVerbose, bool doit) const override;
 
-    eckit::DataHandle *getCachedHandle( const eckit::PathName &path ) const;
+    eckit::DataHandle* getCachedHandle( const eckit::PathName& path ) const;
     void closeDataHandles();
-    eckit::DataHandle *createFileHandle(const eckit::PathName &path);
-    eckit::DataHandle *createAsyncHandle(const eckit::PathName &path);
-    eckit::DataHandle *createDataHandle(const eckit::PathName &path);
-    eckit::DataHandle& getDataHandle( const eckit::PathName &path );
-    eckit::PathName generateDataPath(const Key &key) const;
-    eckit::PathName getDataPath(const Key &key) const;
+    std::unique_ptr<eckit::DataHandle> createFileHandle(const eckit::PathName& path);
+    std::unique_ptr<eckit::DataHandle> createAsyncHandle(const eckit::PathName& path);
+    std::unique_ptr<eckit::DataHandle> createDataHandle(const eckit::PathName& path);
+    eckit::DataHandle& getDataHandle( const eckit::PathName& path );
+    eckit::PathName generateDataPath(const Key& key) const;
+    eckit::PathName getDataPath(const Key& key) const;
     void flushDataHandles();
 
-    void print( std::ostream &out ) const override;
+    void print( std::ostream& out ) const override;
 
 private: // methods
     eckit::URI getAuxiliaryURI(const eckit::URI&, const std::string& ext) const;
 
 private: // types
 
-    typedef std::map< std::string, eckit::DataHandle * >  HandleStore;
+    typedef std::map< std::string, std::unique_ptr<eckit::DataHandle>>  HandleStore;
     typedef std::map< Key, std::string > PathStore;
 
 private: // members
 
+    mutable std::recursive_mutex handlesMutex_;
     HandleStore handles_;    ///< stores the DataHandles being used by the Session
 
     mutable PathStore   dataPaths_;
-
+    size_t archivedFields_;
     std::set<std::string> auxFileExtensions_;
 
 };
