@@ -9,14 +9,11 @@
  */
 
 #include "eckit/option/CmdArgs.h"
-#include "eckit/types/Date.h"
+#include "eckit/option/SimpleOption.h"
 
 #include "fdb5/LibFdb5.h"
-#include "fdb5/rules/Schema.h"
 #include "fdb5/tools/FDBTool.h"
 
-using eckit::Log;
-
 namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -34,7 +31,9 @@ static void usage(const std::string& tool) {
 }
 
 void FDBTool::run() {
-    options_.push_back(new eckit::option::SimpleOption<std::string>("config", "FDB configuration filename"));
+    if(needsConfig_) {
+        options_.push_back(new eckit::option::SimpleOption<std::string>("config", "FDB configuration filename"));
+    }
 
     eckit::option::CmdArgs args(&fdb5::usage, options_, numberOfPositionalArguments(),
                                 minimumPositionalArguments());
