@@ -11,23 +11,19 @@
 #include "eckit/option/CmdArgs.h"
 
 #include "fdb5/toc/TocHandler.h"
-#include "fdb5/toc/TocIndex.h"
 #include "fdb5/tools/FDBTool.h"
 
-#include "fdb5/toc/BTreeIndex.h"
-#include "fdb5/database/Index.h"
-
 #include <string>
 
 using namespace eckit;
 
 //----------------------------------------------------------------------------------------------------------------------
 
-class FDBDumpToc : public fdb5::FDBTool {
+class FDBDumpIndex : public fdb5::FDBTool {
 
   public: // methods
 
-    FDBDumpToc(int argc, char **argv) :
+    FDBDumpIndex(int argc, char **argv) :
         fdb5::FDBTool(argc, argv) {}
 
   private: // methods
@@ -36,14 +32,14 @@ class FDBDumpToc : public fdb5::FDBTool {
     virtual void execute(const option::CmdArgs& args);
 };
 
-void FDBDumpToc::usage(const std::string &tool) const {
+void FDBDumpIndex::usage(const std::string &tool) const {
     Log::info() << std::endl
                 << "Usage: " << tool << " [path1] [path2] ..." << std::endl;
     fdb5::FDBTool::usage(tool);
 }
 
 
-void FDBDumpToc::execute(const option::CmdArgs& args) {
+void FDBDumpIndex::execute(const option::CmdArgs& args) {
 
     // n.b. We don't just open the toc, then check if in the list of indexes, as there
     //      is no reason to think that the indexes map to the same toc (or directory).
@@ -62,7 +58,7 @@ void FDBDumpToc::execute(const option::CmdArgs& args) {
 //----------------------------------------------------------------------------------------------------------------------
 
 int main(int argc, char **argv) {
-    FDBDumpToc app(argc, argv);
+    FDBDumpIndex app(argc, argv);
     return app.start();
 }
 
