@@ -8,20 +8,27 @@
  * does it submit to any jurisdiction.
  */
 
+#include <map>
+#include <memory>
+#include <string>
 #include <unordered_set>
+#include <utility>
+#include <vector>
 
-#include "eckit/option/CmdArgs.h"
-#include "eckit/config/Resource.h"
-#include "eckit/option/SimpleOption.h"
-#include "eckit/option/VectorOption.h"
-#include "eckit/option/CmdArgs.h"
+#include "eckit/exception/Exceptions.h"
 #include "eckit/log/JSON.h"
+#include "eckit/log/Log.h"
+#include "eckit/option/CmdArgs.h"
+#include "eckit/option/SimpleOption.h"
 
+#include "fdb5/LibFdb5.h"
 #include "metkit/hypercube/HyperCube.h"
+#include "metkit/mars/MarsRequest.h"
 
 #include "fdb5/api/FDB.h"
 #include "fdb5/api/helpers/FDBToolRequest.h"
-#include "fdb5/database/DB.h"
+#include "fdb5/api/helpers/ListElement.h"
+#include "fdb5/api/helpers/ListIterator.h"
 #include "fdb5/database/Index.h"
 #include "fdb5/rules/Schema.h"
 #include "fdb5/tools/FDBVisitTool.h"
@@ -29,88 +36,82 @@
 using namespace eckit;
 using namespace eckit::option;
 
-namespace fdb5 {
-namespace tools {
+namespace fdb5::tools {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-class FDBList : public FDBVisitTool {
-
-  public: // methods
-
-    FDBList(int argc, char **argv) :
-        FDBVisitTool(argc, argv, "class,expver"),
-        location_(false),
-        timestamp_(false),
-        length_(false),
-        full_(false),
-        porcelain_(false),
-        json_(false) {
-
+class FDBList: public FDBVisitTool {
+public:  // methods
+    FDBList(int argc, char** argv): FDBVisitTool(argc, argv, "class,expver") {
         options_.push_back(new SimpleOption<bool>("location", "Also print the location of each field"));
         options_.push_back(new SimpleOption<bool>("timestamp", "Also print the timestamp when the field was indexed"));
         options_.push_back(new SimpleOption<bool>("length", "Also print the field size"));
         options_.push_back(new SimpleOption<bool>("full", "Include all entries (including masked duplicates)"));
-        options_.push_back(new SimpleOption<bool>("porcelain", "Streamlined and stable output for input into other tools"));
+        options_.push_back(
+            new SimpleOption<bool>("porcelain",
+                                   "Streamlined and stable output. Useful as input for other tools or scripts."
+                                   "Incompatible with options: location, timestamp, and length"));
         options_.push_back(new SimpleOption<bool>("json", "Output available fields in JSON form"));
         options_.push_back(new SimpleOption<bool>("compact", "Aggregate available fields in MARS requests"));
+        options_.push_back(new SimpleOption<long>("depth", "Output entries up to 'depth' levels deep [1-3]"));
     }
 
-  private: // methods
+private:  // methods
+    void execute(const CmdArgs& args) override;
+    void init(const CmdArgs& args) override;
 
-    virtual void execute(const CmdArgs& args);
-    virtual void init(const CmdArgs &args);
-
-    bool location_;
-    bool timestamp_;
-    bool length_;
-    bool full_;
-    bool porcelain_;
-    bool json_;
-    bool compact_;
+    bool location_ {false};
+    bool timestamp_ {false};
+    bool length_ {false};
+    bool full_ {false};
+    bool porcelain_ {false};
+    bool json_ {false};
+    bool compact_ {false};
+    int  depth_ {3};
 };
 
+//----------------------------------------------------------------------------------------------------------------------
 
-std::string keySignature(const fdb5::Key& key) {
+static std::string keySignature(const fdb5::Key& key) {
     std::string signature;
     std::string separator;
-    for (auto k : key.keys()) {
-        signature += separator+k;
-        separator=":";
+    for (auto&& k : key.keys()) {
+        signature += separator + k;
+        separator  = ":";
     }
     return signature;
 }
 
-
 void FDBList::init(const CmdArgs& args) {
 
     FDBVisitTool::init(args);
 
-    location_ = args.getBool("location", false);
-    timestamp_ = args.getBool("timestamp", false);
-    length_ = args.getBool("length", false);
-    full_ = args.getBool("full", false);
-    porcelain_ = args.getBool("porcelain", false);
-    json_ = args.getBool("json", false);
-    compact_ = args.getBool("compact", false);
+    location_  = args.getBool("location", location_);
+    timestamp_ = args.getBool("timestamp", timestamp_);
+    length_    = args.getBool("length", length_);
+    full_      = args.getBool("full", full_);
+    porcelain_ = args.getBool("porcelain", porcelain_);
+    json_      = args.getBool("json", json_);
+    compact_   = args.getBool("compact", compact_);
+    depth_     = args.getInt("depth", depth_);
+
+    ASSERT(depth_ > 0 && depth_ < 4);
 
     if (json_) {
+        eckit::Log::debug<LibFdb5>() << "Setting porcelain=true" << '\n';
         porcelain_ = true;
-        if (location_ || timestamp_ || length_) {
-            throw UserError("--json and --location/--timestamp/--length not compatible", Here());
-        }
+    }
+
+    if (porcelain_) {
+        if (location_) { throw UserError("--porcelain and --location are not compatible", Here()); }
+        if (timestamp_) { throw UserError("--porcelain and --timestamp are not compatible", Here()); }
+        if (length_) { throw UserError("--porcelain and --length are not compatible", Here()); }
+        if (compact_) { throw UserError("--porcelain and --compact are not compatible", Here()); }
     }
 
     if (compact_) {
-        if (location_) {
-            throw UserError("--compact and --location are not compatible", Here());
-        } 
-        if (full_) {
-            throw UserError("--compact and --full are not compatible", Here());
-        } 
-        if (porcelain_) {
-            throw UserError("--compact and --porcelain are not compatible", Here());
-        } 
+        if (location_) { throw UserError("--compact and --location are not compatible", Here()); }
+        if (full_) { throw UserError("--compact and --full are not compatible", Here()); }
     }
 
     /// @todo option ignore-errors
@@ -122,7 +123,7 @@ void FDBList::execute(const CmdArgs& args) {
 
     std::unique_ptr<JSON> json;
     if (json_) {
-        json.reset(new JSON(Log::info()));
+        json = std::make_unique<JSON>(Log::info());
         json->startList();
     }
 
@@ -135,14 +136,14 @@ void FDBList::execute(const CmdArgs& args) {
         }
 
         // If --full is supplied, then include all entries including duplicates.
-        auto listObject = fdb.list(request, !full_ && !compact_);
+        auto listObject = fdb.list(request, !full_ && !compact_, depth_);
         std::map<std::string, std::map<std::string, std::pair<metkit::mars::MarsRequest, std::unordered_set<Key>>>> requests;
 
         ListElement elem;
         while (listObject.next(elem)) {
 
             if (compact_) {
-                std::vector<Key> keys = elem.key();
+                const auto& keys = elem.keys();
                 ASSERT(keys.size() == 3);
 
                 std::string treeAxes = keys[0];
@@ -165,19 +166,20 @@ void FDBList::execute(const CmdArgs& args) {
                         it->second.emplace(signature, std::make_pair(keys[2].request(), std::unordered_set<Key>{keys[2]}));
                     }
                 }
-            } else {
-                if (json_) {
-                    (*json) << elem;
-                } else {
-                    if (porcelain_) {
-                        elem.print(Log::info(), location_, false, false);
-                    } else {
-                        elem.print(Log::info(), location_, length_, timestamp_, ", ");
-                    }
-                    Log::info() << std::endl;
-                }
+                continue;
             }
-        }
+
+            // JSON output
+            if (json) {
+                *json << elem;
+                continue;
+            }
+
+            elem.print(Log::info(), location_, length_, timestamp_, ", ");
+            Log::info() << std::endl;
+
+        }  // while
+
         if (compact_) {
             for (const auto& tree: requests) {
                 for (const auto& leaf: tree.second) {
@@ -200,20 +202,17 @@ void FDBList::execute(const CmdArgs& args) {
             }
         }
         // n.b. finding no data is not an error for fdb-list
-    }
 
-    if (json_) {
-        json->endList();
-    }
+    }  // requests
+
+    if (json) { json->endList(); }
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
-} // namespace tools
-} // namespace fdb5
+}  // namespace fdb5::tools
 
 int main(int argc, char **argv) {
     fdb5::tools::FDBList app(argc, argv);
     return app.start();
 }
-
