@@ -8,74 +8,66 @@
  * does it submit to any jurisdiction.
  */
 
-#include "eckit/config/Resource.h"
+#include "eckit/exception/Exceptions.h"
 #include "eckit/option/CmdArgs.h"
+#include "eckit/option/SimpleOption.h"
 #include "eckit/option/VectorOption.h"
 
+#include "fdb5/LibFdb5.h"
 #include "fdb5/api/helpers/FDBToolRequest.h"
 #include "fdb5/config/Config.h"
 #include "fdb5/database/Key.h"
-#include "fdb5/LibFdb5.h"
 #include "fdb5/rules/Schema.h"
-/*#include "fdb5/toc/TocCatalogueWriter.h"
-#include "fdb5/toc/TocDBReader.h"*/
 #include "fdb5/toc/TocEngine.h"
 #include "fdb5/tools/FDBTool.h"
 
-using namespace eckit;
+#include <vector>
+
 using namespace eckit::option;
 
-namespace fdb5 {
-namespace tools {
+namespace fdb5::tools {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-class FdbOverlay : public FDBTool {
+class FdbOverlay: public FDBTool {
 
-public: // methods
-
-    FdbOverlay(int argc, char **argv) :
-        FDBTool(argc, argv),
-        variableKeys_{"class", "expver"},
-        remove_(false),
-        force_(false) {
-        options_.push_back(new VectorOption<std::string>("variable-keys",
-                                                         "The keys that may vary between mounted DBs",
-                                                         0, ","));
+public:  // methods
+    FdbOverlay(int argc, char** argv): FDBTool(argc, argv), variableKeys_ {"class", "expver"} {
+        options_.push_back(
+            new VectorOption<std::string>("variable-keys", "The keys that may vary between mounted DBs", 0, ","));
         options_.push_back(new SimpleOption<bool>("remove", "Remove a previously FDB overlay"));
         options_.push_back(new SimpleOption<bool>("force", "Apply overlay even if target already exists"));
     }
 
-private: // methods
-
-    virtual void init(const option::CmdArgs& args);
-    virtual void execute(const option::CmdArgs& args);
-    virtual void usage(const std::string &tool) const;
-
-private: // members
+private:  // methods
+    virtual void init(const CmdArgs& args);
+    virtual void execute(const CmdArgs& args);
+    virtual void usage(const std::string& tool) const;
 
+private:  // members
     std::vector<std::string> variableKeys_;
-    bool remove_;
-    bool force_;
+
+    bool remove_ {false};
+    bool force_ {false};
 };
 
-void FdbOverlay::usage(const std::string &tool) const {
+void FdbOverlay::usage(const std::string& tool) const {
+
+    eckit::Log::info() << "\nUsage: " << tool << " [options] [source DB request] [target DB request]\n\n\n";
 
-    Log::info() << std::endl
-                << "Usage: " << tool << " [options] [source DB request] [target DB request]" << std::endl
-                << std::endl
-                << std::endl;
     FDBTool::usage(tool);
 }
 
-void FdbOverlay::init(const option::CmdArgs& args) {
+void FdbOverlay::init(const CmdArgs& args) {
     FDBTool::init(args);
     args.get("variable-keys", variableKeys_);
-    remove_ = args.getBool("remove", false);
-    force_ = args.getBool("force", false);
+    remove_ = args.getBool("remove", remove_);
+    force_  = args.getBool("force", force_);
 }
 
-void FdbOverlay::execute(const option::CmdArgs& args) {
+void FdbOverlay::execute(const CmdArgs& args) {
+
+    const Config conf = config(args);
 
     if (args.count() != 2) {
         usage("fdb-overlay");
@@ -83,43 +75,46 @@ void FdbOverlay::execute(const option::CmdArgs& args) {
     }
 
     auto parsedSource = FDBToolRequest::requestsFromString("domain=g," + args(0), {}, false, "read");
-    auto parsedTarget = FDBToolRequest::requestsFromString("domain=g," + args(1), {}, false, "read");
     ASSERT(parsedSource.size() == 1);
+
+    auto parsedTarget = FDBToolRequest::requestsFromString("domain=g," + args(1), {}, false, "read");
     ASSERT(parsedTarget.size() == 1);
 
     const auto& sourceRequest = parsedSource.front();
-    const auto& targetRequest = parsedTarget.front();
     ASSERT(!sourceRequest.all());
+
+    const auto& targetRequest = parsedTarget.front();
     ASSERT(!targetRequest.all());
 
-    Config conf = config(args);
-    const Schema& schema = conf.schema();
+    const auto sources = conf.schema().expandDatabase(sourceRequest.request());
+    ASSERT(!sources.empty());
 
-    InspectionKey source;
-    InspectionKey target;
-    ASSERT(schema.expandFirstLevel(sourceRequest.request(), source));
-    ASSERT(schema.expandFirstLevel(targetRequest.request(), target));
+    const auto targets = conf.schema().expandDatabase(targetRequest.request());
+    ASSERT(!targets.empty());
+
+    const auto& source = sources.front();
+    const auto& target = targets.front();
 
     if (remove_) {
-        Log::info() << "Removing " << source << " from " << target << std::endl;
+        eckit::Log::info() << "Removing " << source << " from " << target << std::endl;
     } else {
-        Log::info() << "Applying " << source << " onto " << target << std::endl;
+        eckit::Log::info() << "Applying " << source << " onto " << target << std::endl;
     }
 
     if (source.keys() != target.keys()) {
         std::stringstream ss;
         ss << "Keys insufficiently matching for mount: " << source << " : " << target << std::endl;
-        throw UserError(ss.str(), Here());
+        throw eckit::UserError(ss.str(), Here());
     }
 
     std::set<std::string> vkeys(variableKeys_.begin(), variableKeys_.end());
     for (const auto& kv : target) {
-        auto it = source.find(kv.first);
-        ASSERT(it != source.end());
+        const auto [it, found] = source.find(kv.first);
+        ASSERT(found);
         if (kv.second != it->second && vkeys.find(kv.first) == vkeys.end()) {
             std::stringstream ss;
             ss << "Key " << kv.first << " not allowed to differ between DBs: " << source << " : " << target;
-            throw UserError(ss.str(), Here());
+            throw eckit::UserError(ss.str(), Here());
         }
     }
 
@@ -127,13 +122,13 @@ void FdbOverlay::execute(const option::CmdArgs& args) {
     if (!dbSource->exists()) {
         std::stringstream ss;
         ss << "Source database not found: " << source << std::endl;
-        throw UserError(ss.str(), Here());
+        throw eckit::UserError(ss.str(), Here());
     }
 
     if (dbSource->type() != TocEngine::typeName()) {
         std::stringstream ss;
         ss << "Only TOC DBs currently supported" << std::endl;
-        throw UserError(ss.str(), Here());
+        throw eckit::UserError(ss.str(), Here());
     }
 
     std::unique_ptr<CatalogueReader> dbTarget = CatalogueReaderFactory::instance().build(target, conf);
@@ -142,7 +137,7 @@ void FdbOverlay::execute(const option::CmdArgs& args) {
         if (!dbTarget->exists()) {
             std::stringstream ss;
             ss << "Target database must already exist: " << target << std::endl;
-            throw UserError(ss.str(), Here());
+            throw eckit::UserError(ss.str(), Here());
         }
     } else {
         if (dbTarget->exists() && !force_) {
@@ -150,7 +145,7 @@ void FdbOverlay::execute(const option::CmdArgs& args) {
             ss << "Target database already exists: " << target << std::endl;
             eckit::Log::error() << ss.str() << std::endl;
             eckit::Log::error() << "To mount to existing target, rerun with --force" << std::endl;
-            throw UserError(ss.str(), Here());
+            throw eckit::UserError(ss.str(), Here());
         }
     }
 
@@ -164,8 +159,7 @@ void FdbOverlay::execute(const option::CmdArgs& args) {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-} // namespace tools
-} // namespace fbb5
+}  // namespace fdb5::tools
 
 int main(int argc, char **argv) {
     fdb5::tools::FdbOverlay app(argc, argv);
