@@ -9,19 +9,15 @@
  */
 
 #include "fdb5/types/Type.h"
+#include "eckit/utils/StringTools.h"
 #include "metkit/mars/MarsRequest.h"
 
 namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-Type::Type(const std::string &name, const std::string &type) :
-    name_(name),
-    type_(type) {
-}
-
-Type::~Type() {
-}
+Type::Type(const std::string& name, const std::string& type, const std::string& alias) :
+    name_(name), type_(type), alias_(alias) { }
 
 void Type::getValues(const metkit::mars::MarsRequest &request,
                      const std::string &keyword,
@@ -31,18 +27,20 @@ void Type::getValues(const metkit::mars::MarsRequest &request,
     request.getValues(keyword, values, true);
 }
 
-const std::string &Type::type() const {
+const std::string& Type::alias() const {
+    return alias_.empty() ? name_ : alias_;
+}
+
+const std::string& Type::type() const {
     return type_;
 }
 
-std::string Type::tidy(const std::string&,
-                       const std::string &value) const  {
-    return value;
+std::string Type::tidy(const std::string &value) const  {
+    return eckit::StringTools::lower(value);
 }
 
-std::string Type::toKey(const std::string&,
-                        const std::string &value) const {
-    return value;
+std::string Type::toKey(const std::string &value) const {
+    return eckit::StringTools::lower(value);
 }
 
 std::ostream &operator<<(std::ostream &s, const Type &x) {
