@@ -40,15 +40,15 @@ class Type : private eckit::NonCopyable {
 
 public: // methods
 
-    Type(const std::string &name, const std::string &type);
+    Type(const std::string &name, const std::string& type, const std::string& alias = "");
 
-    virtual ~Type();
+    virtual ~Type() = default;
 
-    virtual std::string tidy(const std::string &keyword,
-                             const std::string &value) const ;
+    const std::string& alias() const;
+    
+    virtual std::string tidy(const std::string &value) const ;
 
-    virtual std::string toKey(const std::string &keyword,
-                              const std::string &value) const ;
+    virtual std::string toKey(const std::string &value) const ;
 
     virtual void getValues(const metkit::mars::MarsRequest &request,
                            const std::string &keyword,
@@ -74,6 +74,7 @@ protected: // members
 
     std::string name_;
     std::string type_;
+    std::string alias_;
 
 };
 
