@@ -28,14 +28,13 @@ public: // methods
 
     TypeExpver(const std::string &name, const std::string &type);
 
-    virtual ~TypeExpver() override;
+    ~TypeExpver() override;
 
-    virtual std::string tidy(const std::string &keyword,
-                             const std::string &value) const override;
+    std::string tidy(const std::string &value) const override;
 
 private: // methods
 
-    virtual void print( std::ostream &out ) const override;
+    void print( std::ostream &out ) const override;
 
 };
 
