@@ -28,12 +28,11 @@ public: // methods
 
     TypeGrid(const std::string &name, const std::string &type);
 
-    virtual ~TypeGrid() override;
+    ~TypeGrid() override;
 
 private: // methods
 
-    virtual std::string toKey(const std::string& keyword,
-                              const std::string& value) const override;
+    std::string toKey(const std::string& value) const override;
 
     virtual void getValues(const metkit::mars::MarsRequest &request,
                            const std::string &keyword,
@@ -41,7 +40,7 @@ private: // methods
                            const Notifier &wind,
                            const CatalogueReader* cat) const override;
 
-    virtual void print( std::ostream &out ) const override;
+    void print( std::ostream &out ) const override;
 
 };
 
