@@ -28,7 +28,7 @@ public: // methods
 
     TypeIgnore(const std::string &name, const std::string &type);
 
-    virtual ~TypeIgnore() override;
+    ~TypeIgnore() override;
 
     virtual void getValues(const metkit::mars::MarsRequest &request,
                            const std::string &keyword,
@@ -36,12 +36,11 @@ public: // methods
                            const Notifier &wind,
                            const CatalogueReader* cat) const override;
 
-    virtual std::string toKey(const std::string& keyword,
-                              const std::string& value) const override;
+    std::string toKey(const std::string& value) const override;
 
 private: // methods
 
-    virtual void print( std::ostream &out ) const override;
+    void print( std::ostream &out ) const override;
 
 };
 
