@@ -9,6 +9,9 @@
  */
 
 // #include "eckit/exception/Exceptions.h"
+#include <iomanip>
+#include <sstream>
+
 #include "eckit/utils/Translator.h"
 #include "eckit/types/Date.h"
 
@@ -22,18 +25,20 @@ namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
-TypeMonth::TypeMonth(const std::string &name, const std::string &type) :
-    Type(name, type) {
+TypeMonth::TypeMonth(const std::string& name, const std::string& type, const std::string& alias) :
+    Type(name, type, alias) {
 }
 
 TypeMonth::~TypeMonth() {
 }
 
-std::string TypeMonth::toKey(const std::string&,
-                             const std::string& value) const {
+std::string TypeMonth::toKey(const std::string& value) const {
 
     eckit::Date date(value);
-    return std::to_string(date.year() * 100 + date.month());
+
+    std::ostringstream ss;
+    ss << std::setw(2) << std::setfill('0') << date.month();
+    return ss.str();
 }
 
 void TypeMonth::getValues(const metkit::mars::MarsRequest& request,
