@@ -26,12 +26,11 @@ class TypeMonth : public Type {
 
 public: // methods
 
-    TypeMonth(const std::string &name, const std::string &type);
+    TypeMonth(const std::string& name, const std::string& type, const std::string& alias = "month");
 
-    virtual ~TypeMonth() override;
+    ~TypeMonth() override;
 
-    virtual std::string toKey(const std::string& keyword,
-                              const std::string& value) const override;
+    std::string toKey(const std::string& value) const override;
 
     virtual void getValues(const metkit::mars::MarsRequest &request,
                            const std::string &keyword,
@@ -41,7 +40,7 @@ public: // methods
 
 private: // methods
 
-    virtual void print( std::ostream &out ) const override;
+    void print( std::ostream &out ) const override;
 
 };
 
