@@ -13,7 +13,7 @@
 #include "metkit/mars/ParamID.h"
 #include "metkit/mars/Param.h"
 
-#include "fdb5/database/DB.h"
+#include "fdb5/database/Catalogue.h"
 #include "fdb5/database/Notifier.h"
 #include "fdb5/types/TypeParam.h"
 #include "fdb5/types/TypesFactory.h"
@@ -35,16 +35,16 @@ void TypeParam::getValues(const metkit::mars::MarsRequest &request,
                           const std::string &keyword,
                           eckit::StringList &values,
                           const Notifier &wind,
-                          const DB *db) const {
-    ASSERT(db);
+                          const CatalogueReader* cat) const {
+    ASSERT(cat);
 
-    eckit::StringSet ax;
+    eckit::DenseSet<std::string> ax;
 
-    db->axis(keyword, ax);
+    cat->axis(keyword, ax);
 
     eckit::StringList us;
 
-    Type::getValues(request, keyword, us, wind, db);
+    Type::getValues(request, keyword, us, wind, cat);
 
     std::vector<Param> user;
     std::copy(us.begin(), us.end(), std::back_inserter(user));
