@@ -8,6 +8,7 @@
  * does it submit to any jurisdiction.
  */
 
+#include "eckit/container/DenseSet.h"
 #include "eckit/utils/Translator.h"
 
 #include "metkit/mars/MarsRequest.h"
@@ -17,6 +18,7 @@
 #include "fdb5/types/TypesFactory.h"
 #include "fdb5/types/TypeStep.h"
 #include "fdb5/database/Catalogue.h"
+#include <string>
 
 using metkit::mars::StepRange;
 using metkit::mars::StepRangeNormalise;
@@ -34,8 +36,7 @@ TypeStep::~TypeStep() {
 }
 
 
-std::string TypeStep::toKey(const std::string&,
-                            const std::string& value) const {
+std::string TypeStep::toKey(const std::string& value) const {
     return StepRange(value);
 }
 
@@ -72,7 +73,7 @@ void TypeStep::getValues(const metkit::mars::MarsRequest& request,
 
         // Get the axis
 
-        eckit::StringSet ax;
+        eckit::DenseSet<std::string> ax;
         cat->axis("step", ax);
 
         std::vector<StepRange> axis;
